# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

MRISP_PAINT_METADATA = Metadata(
    id="ee10714d3437e3ff63bdc4e23471b26ed9cb9121.boutiques",
    name="mrisp_paint",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class MrispPaintOutputs(typing.NamedTuple):
    """
    Output object returned when calling `mrisp_paint(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_file: OutputPathType
    """Output file containing a surface-worth of per-vertex values in
    'curvature' format."""


def mrisp_paint(
    template_file: InputPathType,
    input_surface: InputPathType,
    output_name: str,
    subjects_dir: str | None = None,
    vertex_coords: str | None = None,
    average_flag: float | None = None,
    normalize_flag: bool = False,
    frame_number: float | None = None,
    square_root_flag: bool = False,
    variance_params: str | None = None,
    usage_flag: bool = False,
    birn_info_flag: bool = False,
    help_flag: bool = False,
    diag_vertex: float | None = None,
    version_flag: bool = False,
    diag_write_flag: bool = False,
    runner: Runner | None = None,
) -> MrispPaintOutputs:
    """
    A tool for extracting arrays from a surface-registration template file and
    sampling them onto a surface mesh.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        template_file: Full path to the template file. Template may contain\
            multiple parameters. Example: 'somepath/mytemplate.tif#1'.
        input_surface: Full path to the input surface file, which provides the\
            grid onto which the template data is sampled.
        output_name: Output file name. Saves the surface-worth of per-vertex\
            values.
        subjects_dir: Set the SUBJECTS_DIR. Default: use environment variable.
        vertex_coords: Treat overlay as a surface and write it into a 3 frame\
            parameterization.
        average_flag: Average curvature patterns a given number of times.
        normalize_flag: Normalize curvature by variance.
        frame_number: Paint the specified frame number to the output file.\
            Default: 0.
        square_root_flag: Take the square-root of the output variable.
        variance_params: Generate variance map. Requires subject name,\
            hemisphere, and field number.
        usage_flag: Print usage.
        birn_info_flag: Print BIRN-standard program information.
        help_flag: Print help message.
        diag_vertex: Invoke diagnostics for a specific vertex number.
        version_flag: Print version information.
        diag_write_flag: Write some diagnostics (DIAG_WRITE).
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MrispPaintOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(MRISP_PAINT_METADATA)
    cargs = []
    cargs.append("mrisp_paint")
    cargs.append(execution.input_file(template_file))
    cargs.append(execution.input_file(input_surface))
    cargs.append(output_name)
    if subjects_dir is not None:
        cargs.extend([
            "-SDIR",
            subjects_dir
        ])
    if vertex_coords is not None:
        cargs.extend([
            "-coords",
            vertex_coords
        ])
    if average_flag is not None:
        cargs.extend([
            "-A",
            str(average_flag)
        ])
    if normalize_flag:
        cargs.append("-N")
    if frame_number is not None:
        cargs.extend([
            "-f",
            str(frame_number)
        ])
    if square_root_flag:
        cargs.append("-S")
    if variance_params is not None:
        cargs.extend([
            "-variance",
            variance_params
        ])
    if usage_flag:
        cargs.append("-?")
    if birn_info_flag:
        cargs.append("--all-info")
    if help_flag:
        cargs.append("--help")
    if diag_vertex is not None:
        cargs.extend([
            "-V",
            str(diag_vertex)
        ])
    if version_flag:
        cargs.append("--version")
    if diag_write_flag:
        cargs.append("-W")
    ret = MrispPaintOutputs(
        root=execution.output_file("."),
        output_file=execution.output_file(output_name),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "MRISP_PAINT_METADATA",
    "MrispPaintOutputs",
    "mrisp_paint",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

V__SHIFT_VOLUME_METADATA = Metadata(
    id="8632323dd36a0ba5f059c998ffcd7826b058d4be.boutiques",
    name="@Shift_Volume",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class VShiftVolumeOutputs(typing.NamedTuple):
    """
    Output object returned when calling `v__shift_volume(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_file: OutputPathType
    """Shifted output dataset."""


def v__shift_volume(
    dset: InputPathType,
    prefix: str,
    rai_shift_vector: list[float] | None = None,
    mni_anat_to_mni: bool = False,
    mni_to_mni_anat: bool = False,
    no_cp: bool = False,
    runner: Runner | None = None,
) -> VShiftVolumeOutputs:
    """
    Tool to shift a dataset in the RAI coordinate system or between MNI anatomical
    space and MNI space.
    
    Author: AFNI Developers
    
    URL: https://afni.nimh.nih.gov/
    
    Args:
        dset: Input dataset, typically an anatomical dataset to be aligned to\
            BASE.
        prefix: Prefix for the output dataset.
        rai_shift_vector: Move dataset by dR, dA, dI mm (RAI coordinate system).
        mni_anat_to_mni: Move dataset from MNI Anatomical space to MNI space\
            (equivalent to -rai_shift 0 -4 -5).
        mni_to_mni_anat: Move dataset from MNI space to MNI Anatomical space\
            (equivalent to -rai_shift 0 4 5).
        no_cp: Do not create new data, shift the existing ones (use with\
            caution).
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VShiftVolumeOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(V__SHIFT_VOLUME_METADATA)
    cargs = []
    cargs.append("@Shift_Volume")
    if rai_shift_vector is not None:
        cargs.extend([
            "-rai_shift",
            *map(str, rai_shift_vector)
        ])
    if mni_anat_to_mni:
        cargs.append("-MNI_Anat_to_MNI")
    if mni_to_mni_anat:
        cargs.append("-MNI_to_MNI_Anat")
    cargs.extend([
        "-dset",
        execution.input_file(dset)
    ])
    if no_cp:
        cargs.append("-no_cp")
    cargs.extend([
        "-prefix",
        prefix
    ])
    ret = VShiftVolumeOutputs(
        root=execution.output_file("."),
        output_file=execution.output_file(prefix + ".nii.gz"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "VShiftVolumeOutputs",
    "V__SHIFT_VOLUME_METADATA",
    "v__shift_volume",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

CIFTI_CREATE_LABEL_METADATA = Metadata(
    id="2de4152aa98a5a2ba317600135a0292a6a69deee.boutiques",
    name="cifti-create-label",
    package="workbench",
    container_image_tag="brainlife/connectome_workbench:1.5.0-freesurfer-update",
)


@dataclasses.dataclass
class CiftiCreateLabelVolume:
    """
    volume component.
    """
    label_volume: InputPathType
    """label volume file containing the data to be copied"""
    structure_label_volume: InputPathType
    """label volume file that defines which voxels to use"""
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        cargs.append("-volume")
        cargs.append(execution.input_file(self.label_volume))
        cargs.append(execution.input_file(self.structure_label_volume))
        return cargs


@dataclasses.dataclass
class CiftiCreateLabelLeftLabel:
    """
    label file for left surface.
    """
    label: InputPathType
    """the label file"""
    opt_roi_left_roi_metric: InputPathType | None = None
    """roi of vertices to use from left surface: the ROI as a metric file"""
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        cargs.append("-left-label")
        cargs.append(execution.input_file(self.label))
        if self.opt_roi_left_roi_metric is not None:
            cargs.extend([
                "-roi-left",
                execution.input_file(self.opt_roi_left_roi_metric)
            ])
        return cargs


@dataclasses.dataclass
class CiftiCreateLabelRightLabel:
    """
    label for left surface.
    """
    label: InputPathType
    """the label file"""
    opt_roi_right_roi_metric: InputPathType | None = None
    """roi of vertices to use from right surface: the ROI as a metric file"""
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        cargs.append("-right-label")
        cargs.append(execution.input_file(self.label))
        if self.opt_roi_right_roi_metric is not None:
            cargs.extend([
                "-roi-right",
                execution.input_file(self.opt_roi_right_roi_metric)
            ])
        return cargs


@dataclasses.dataclass
class CiftiCreateLabelCerebellumLabel:
    """
    label for the cerebellum.
    """
    label: InputPathType
    """the label file"""
    opt_roi_cerebellum_roi_metric: InputPathType | None = None
    """roi of vertices to use from right surface: the ROI as a metric file"""
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        cargs.append("-cerebellum-label")
        cargs.append(execution.input_file(self.label))
        if self.opt_roi_cerebellum_roi_metric is not None:
            cargs.extend([
                "-roi-cerebellum",
                execution.input_file(self.opt_roi_cerebellum_roi_metric)
            ])
        return cargs


class CiftiCreateLabelOutputs(typing.NamedTuple):
    """
    Output object returned when calling `cifti_create_label(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    cifti_out: OutputPathType
    """the output cifti file"""


def cifti_create_label(
    cifti_out: str,
    volume: CiftiCreateLabelVolume | None = None,
    left_label: CiftiCreateLabelLeftLabel | None = None,
    right_label: CiftiCreateLabelRightLabel | None = None,
    cerebellum_label: CiftiCreateLabelCerebellumLabel | None = None,
    runner: Runner | None = None,
) -> CiftiCreateLabelOutputs:
    """
    Create a cifti label file.
    
    All input files must have the same number of columns/subvolumes. Only the
    specified components will be in the output cifti. At least one component
    must be specified.
    
    The -volume option requires two volume arguments, the label-volume argument
    contains all labels you want to display (e.g. nuclei of the thalamus),
    whereas the structure-label-volume argument contains all CIFTI voxel-based
    structures you want to include data within (e.g. THALAMUS_LEFT,
    THALAMUS_RIGHT, etc). See -volume-label-import and -volume-help for format
    details of label volume files. If you just want the labels in voxels to be
    the structure names, you may use the same file for both arguments. The
    structure-label-volume must use some of the label names from this list, all
    other label names in the structure-label-volume will be ignored:
    
    CORTEX_LEFT
    CORTEX_RIGHT
    CEREBELLUM
    ACCUMBENS_LEFT
    ACCUMBENS_RIGHT
    ALL_GREY_MATTER
    ALL_WHITE_MATTER
    AMYGDALA_LEFT
    AMYGDALA_RIGHT
    BRAIN_STEM
    CAUDATE_LEFT
    CAUDATE_RIGHT
    CEREBELLAR_WHITE_MATTER_LEFT
    CEREBELLAR_WHITE_MATTER_RIGHT
    CEREBELLUM_LEFT
    CEREBELLUM_RIGHT
    CEREBRAL_WHITE_MATTER_LEFT
    CEREBRAL_WHITE_MATTER_RIGHT
    CORTEX
    DIENCEPHALON_VENTRAL_LEFT
    DIENCEPHALON_VENTRAL_RIGHT
    HIPPOCAMPUS_LEFT
    HIPPOCAMPUS_RIGHT
    INVALID
    OTHER
    OTHER_GREY_MATTER
    OTHER_WHITE_MATTER
    PALLIDUM_LEFT
    PALLIDUM_RIGHT
    PUTAMEN_LEFT
    PUTAMEN_RIGHT
    THALAMUS_LEFT
    THALAMUS_RIGHT.
    
    Author: Connectome Workbench Developers
    
    URL: https://github.com/Washington-University/workbench
    
    Args:
        cifti_out: the output cifti file.
        volume: volume component.
        left_label: label file for left surface.
        right_label: label for left surface.
        cerebellum_label: label for the cerebellum.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CiftiCreateLabelOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(CIFTI_CREATE_LABEL_METADATA)
    cargs = []
    cargs.append("wb_command")
    cargs.append("-cifti-create-label")
    cargs.append(cifti_out)
    if volume is not None:
        cargs.extend(volume.run(execution))
    if left_label is not None:
        cargs.extend(left_label.run(execution))
    if right_label is not None:
        cargs.extend(right_label.run(execution))
    if cerebellum_label is not None:
        cargs.extend(cerebellum_label.run(execution))
    ret = CiftiCreateLabelOutputs(
        root=execution.output_file("."),
        cifti_out=execution.output_file(cifti_out),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "CIFTI_CREATE_LABEL_METADATA",
    "CiftiCreateLabelCerebellumLabel",
    "CiftiCreateLabelLeftLabel",
    "CiftiCreateLabelOutputs",
    "CiftiCreateLabelRightLabel",
    "CiftiCreateLabelVolume",
    "cifti_create_label",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

V__DJUNCT_OVERLAP_CHECK_METADATA = Metadata(
    id="61eef1d6dc1535a1dc4452ab3c3b8b5da02a1e6d.boutiques",
    name="@djunct_overlap_check",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class VDjunctOverlapCheckOutputs(typing.NamedTuple):
    """
    Output object returned when calling `v__djunct_overlap_check(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def v__djunct_overlap_check(
    ulay: InputPathType,
    olay: InputPathType,
    prefix: str,
    box_focus_slices: InputPathType | None = None,
    montgap: float | None = None,
    montcolor: str | None = None,
    cbar: str | None = None,
    opacity: float | None = None,
    zerocolor: str | None = None,
    set_dicom_xyz: list[float] | None = None,
    ulay_range: list[float] | None = None,
    ulay_range_nz: list[float] | None = None,
    montx: float | None = None,
    monty: float | None = None,
    montx_cat: float | None = None,
    monty_cat: float | None = None,
    label_mode: str | None = None,
    pbar_posonly_off: bool = False,
    edgy_ulay: bool = False,
    set_dicom_xyz_off: bool = False,
    no_cor: bool = False,
    no_axi: bool = False,
    no_sag: bool = False,
    no_clean: bool = False,
    runner: Runner | None = None,
) -> VDjunctOverlapCheckOutputs:
    """
    A helper script for visualizing overlap between datasets in AFNI.
    
    Author: AFNI Developers
    
    URL: https://afni.nimh.nih.gov/
    
    Args:
        ulay: Dataset to use as the underlay (background).
        olay: Dataset to use as the overlay (foreground).
        prefix: Prefix for the output files.
        box_focus_slices: Dataset for box focus slices.
        montgap: Gap between montage slices.
        montcolor: Color of the montage gap.
        cbar: Colorbar for the overlay.
        opacity: Opacity of the overlay.
        zerocolor: Color for zero values.
        set_dicom_xyz: Set DICOM coordinates for slice location.
        ulay_range: Range for underlay values.
        ulay_range_nz: Range for non-zero underlay values.
        montx: Number of panels in X direction in montage.
        monty: Number of panels in Y direction in montage.
        montx_cat: Number of X panes per image in montage.
        monty_cat: Number of Y panes per image in montage.
        label_mode: Label mode.
        pbar_posonly_off: Turn off position-only p-bar.
        edgy_ulay: Edgify the underlay.
        set_dicom_xyz_off: Turn off DICOM coordinates setting.
        no_cor: Skip coronal slices.
        no_axi: Skip axial slices.
        no_sag: Skip sagittal slices.
        no_clean: Do not clean up temporary files.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VDjunctOverlapCheckOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(V__DJUNCT_OVERLAP_CHECK_METADATA)
    cargs = []
    cargs.append("@djunct_overlap_check")
    cargs.append(execution.input_file(ulay))
    cargs.append(execution.input_file(olay))
    cargs.append(prefix)
    if box_focus_slices is not None:
        cargs.append(execution.input_file(box_focus_slices))
    if montgap is not None:
        cargs.extend([
            "-montgap",
            str(montgap)
        ])
    if montcolor is not None:
        cargs.extend([
            "-montcolor",
            montcolor
        ])
    if cbar is not None:
        cargs.extend([
            "-cbar",
            cbar
        ])
    if opacity is not None:
        cargs.extend([
            "-opacity",
            str(opacity)
        ])
    if zerocolor is not None:
        cargs.extend([
            "-zerocolor",
            zerocolor
        ])
    if set_dicom_xyz is not None:
        cargs.extend([
            "-set_dicom_xyz",
            *map(str, set_dicom_xyz)
        ])
    if ulay_range is not None:
        cargs.extend([
            "-ulay_range",
            *map(str, ulay_range)
        ])
    if ulay_range_nz is not None:
        cargs.extend([
            "-ulay_range_nz",
            *map(str, ulay_range_nz)
        ])
    if montx is not None:
        cargs.extend([
            "-montx",
            str(montx)
        ])
    if monty is not None:
        cargs.extend([
            "-monty",
            str(monty)
        ])
    if montx_cat is not None:
        cargs.extend([
            "-montx_cat",
            str(montx_cat)
        ])
    if monty_cat is not None:
        cargs.extend([
            "-monty_cat",
            str(monty_cat)
        ])
    if label_mode is not None:
        cargs.extend([
            "-label_mode",
            label_mode
        ])
    if pbar_posonly_off:
        cargs.append("-pbar_posonly_off")
    if edgy_ulay:
        cargs.append("-edgy_ulay")
    if set_dicom_xyz_off:
        cargs.append("-set_dicom_xyz_off")
    if no_cor:
        cargs.append("-no_cor")
    if no_axi:
        cargs.append("-no_axi")
    if no_sag:
        cargs.append("-no_sag")
    if no_clean:
        cargs.append("-no_clean")
    ret = VDjunctOverlapCheckOutputs(
        root=execution.output_file("."),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "VDjunctOverlapCheckOutputs",
    "V__DJUNCT_OVERLAP_CHECK_METADATA",
    "v__djunct_overlap_check",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

ANTS_MOTION_CORR_METADATA = Metadata(
    id="101179d7b402b3245503ed337e9b9c5479b1f966.boutiques",
    name="antsMotionCorr",
    package="ants",
    container_image_tag="antsx/ants:v2.5.3",
)


class AntsMotionCorrOutputs(typing.NamedTuple):
    """
    Output object returned when calling `ants_motion_corr(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_transform_prefix: OutputPathType
    """The output is the transformation matrix."""
    warped_image: OutputPathType
    """The output is the warped moving image."""
    average_image_output: OutputPathType
    """The output is the averaged image of the input time series."""


def ants_motion_corr(
    dimensionality: typing.Literal[2, 3] | None = None,
    n_images: int | None = None,
    metric: str | None = None,
    use_fixed_reference_image: typing.Literal[0, 1] | None = None,
    use_scales_estimator: bool = False,
    transform: str | None = None,
    iterations: str | None = None,
    smoothing_sigmas: str | None = None,
    shrink_factors: str | None = None,
    output: str | None = None,
    average_image: bool = False,
    write_displacement: bool = False,
    use_histogram_matching: typing.Literal[0, 1] | None = None,
    random_seed: int | None = None,
    interpolation: typing.Literal["Linear", "NearestNeighbor", "BSpline", "BlackmanWindowedSinc", "CosineWindowedSinc", "WelchWindowedSinc", "HammingWindowedSinc", "LanczosWindowedSinc"] | None = None,
    verbose: typing.Literal[0, 1] | None = None,
    runner: Runner | None = None,
) -> AntsMotionCorrOutputs:
    """
    ANTS Motion Correction application to perform motion correction on 4D time
    series data.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        dimensionality: This option forces the image to be treated as a\
            specified-dimensional image. If not specified, the program tries to\
            infer the dimensionality from the input image.
        n_images: This option sets the number of images to use to construct the\
            template image.
        metric: Metrics for registration: GC (global correlation), CC (ANTS\
            neighborhood cross correlation), MI (Mutual information), and Demons.
        use_fixed_reference_image: Use a fixed reference image to correct all\
            volumes, instead of correcting each image to the prior volume in the\
            time series.
        use_scales_estimator: Use the scale estimator to control optimization.
        transform: Several transform options are available: Affine, Rigid,\
            GaussianDisplacementField, SyN.
        iterations: Specify the number of iterations at each level.
        smoothing_sigmas: Specify the sigma for smoothing at each level.\
            Smoothing may be specified in mm units or voxels with 'AxBxCmm' or\
            'AxBxCvox'. No units implies voxels.
        shrink_factors: Specify the shrink factor for the virtual domain\
            (typically the fixed image) at each level.
        output: Specify the output transform prefix (output format is .nii.gz\
            ). Optionally, one can choose to warp the moving image to the fixed\
            space and, if the inverse transform exists, one can also output the\
            warped fixed image.
        average_image: Average the input time series image.
        write_displacement: Write the low-dimensional 3D transforms to a 4D\
            displacement field.
        use_histogram_matching: Histogram match the moving images to the\
            reference image.
        random_seed: Use a fixed seed for random number generation.
        interpolation: Several interpolation options are available in ITK. The\
            above are available (default Linear).
        verbose: Verbose output.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `AntsMotionCorrOutputs`).
    """
    if random_seed is not None and not (1 <= random_seed): 
        raise ValueError(f"'random_seed' must be greater than 1 <= x but was {random_seed}")
    runner = runner or get_global_runner()
    execution = runner.start_execution(ANTS_MOTION_CORR_METADATA)
    cargs = []
    cargs.append("antsMotionCorr")
    if dimensionality is not None:
        cargs.extend([
            "--dimensionality",
            str(dimensionality)
        ])
    if n_images is not None:
        cargs.extend([
            "--n-images",
            str(n_images)
        ])
    if metric is not None:
        cargs.extend([
            "--metric",
            metric
        ])
    if use_fixed_reference_image is not None:
        cargs.extend([
            "--useFixedReferenceImage",
            str(use_fixed_reference_image)
        ])
    if use_scales_estimator:
        cargs.append("--useScalesEstimator")
    if transform is not None:
        cargs.extend([
            "--transform",
            transform
        ])
    if iterations is not None:
        cargs.extend([
            "--iterations",
            iterations
        ])
    if smoothing_sigmas is not None:
        cargs.extend([
            "--smoothingSigmas",
            smoothing_sigmas
        ])
    if shrink_factors is not None:
        cargs.extend([
            "--shrinkFactors",
            shrink_factors
        ])
    if output is not None:
        cargs.extend([
            "--output",
            output
        ])
    if average_image:
        cargs.append("--average-image")
    if write_displacement:
        cargs.append("--write-displacement")
    if use_histogram_matching is not None:
        cargs.extend([
            "--use-histogram-matching",
            str(use_histogram_matching)
        ])
    if random_seed is not None:
        cargs.extend([
            "--random-seed",
            str(random_seed)
        ])
    if interpolation is not None:
        cargs.extend([
            "--interpolation",
            interpolation
        ])
    if verbose is not None:
        cargs.extend([
            "--verbose",
            str(verbose)
        ])
    ret = AntsMotionCorrOutputs(
        root=execution.output_file("."),
        output_transform_prefix=execution.output_file("[OUTPUT_TRANSFORM_PREFIX]Affine.mat"),
        warped_image=execution.output_file("[OUTPUT_TRANSFORM_PREFIX]Warped.nii.gz"),
        average_image_output=execution.output_file("[OUTPUT_TRANSFORM_PREFIX]Average.nii.gz"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "ANTS_MOTION_CORR_METADATA",
    "AntsMotionCorrOutputs",
    "ants_motion_corr",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

TRAC_ALL_METADATA = Metadata(
    id="73a5f89795c73c559ddd404cdcdd1e92ea957a5a.boutiques",
    name="trac-all",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class TracAllOutputs(typing.NamedTuple):
    """
    Output object returned when calling `trac_all(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    default_log: OutputPathType
    """Default log file"""
    default_cmd: OutputPathType
    """Default command file"""


def trac_all(
    config_file: InputPathType | None = None,
    subject_name: str | None = None,
    dicom_file: InputPathType | None = None,
    assemble_measures: bool = False,
    no_pathway_priors: bool = False,
    help_: bool = False,
    runner: Runner | None = None,
) -> TracAllOutputs:
    """
    Reconstruct white-matter pathways using an atlas of the underlying anatomy.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        config_file: Configuration file to set analysis options (dmrirc file).
        subject_name: Subject name (if not defined in dmrirc).
        dicom_file: Input DWI DICOM (if not defined in dmrirc).
        assemble_measures: Assemble pathway measures from multiple subjects\
            (step 4).
        no_pathway_priors: Skip pathway priors (step 1.6).
        help_: Print full contents of help.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `TracAllOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(TRAC_ALL_METADATA)
    cargs = []
    cargs.append("trac-all")
    if config_file is not None:
        cargs.extend([
            "-c",
            execution.input_file(config_file)
        ])
    if subject_name is not None:
        cargs.extend([
            "-s",
            subject_name
        ])
    if dicom_file is not None:
        cargs.extend([
            "-i",
            execution.input_file(dicom_file)
        ])
    if assemble_measures:
        cargs.append("-stat")
    if no_pathway_priors:
        cargs.append("-noprior")
    if help_:
        cargs.append("-help")
    ret = TracAllOutputs(
        root=execution.output_file("."),
        default_log=execution.output_file("scripts/trac-all.log"),
        default_cmd=execution.output_file("scripts/trac-all.cmd"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "TRAC_ALL_METADATA",
    "TracAllOutputs",
    "trac_all",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

MRI_CREATE_T2COMBINED_METADATA = Metadata(
    id="629ff11abf5f1e027b0ae35e81e63322a5505789.boutiques",
    name="mri_create_t2combined",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class MriCreateT2combinedOutputs(typing.NamedTuple):
    """
    Output object returned when calling `mri_create_t2combined(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    logfile: OutputPathType
    """Log file generated by the command"""
    t2combined_output: OutputPathType
    """The T2* 7T combined partial-brain halves output file"""


def mri_create_t2combined(
    subjid: str,
    t1wb: InputPathType,
    t2upper: InputPathType,
    t2lower: InputPathType,
    t2combined: str,
    t2middle: InputPathType | None = None,
    show: bool = False,
    runner: Runner | None = None,
) -> MriCreateT2combinedOutputs:
    """
    Utility to combine two or three T2* 7T partial-brain volumes into one volume,
    retaining resolution and registered to the MPRAGE anatomical.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        subjid: Subject ID of the FreeSurfer anatomical.
        t1wb: T1 7T whole-brain image.
        t2upper: T2* 7T partial-brain upper half image.
        t2lower: T2* 7T partial-brain lower half image.
        t2combined: Output file for T2* 7T combined partial-brain halves.
        t2middle: T2* 7T partial-brain middle half image, or enter 'none'.
        show: If the last argument is 'show', then the commands that will run\
            are shown but not executed.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MriCreateT2combinedOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(MRI_CREATE_T2COMBINED_METADATA)
    cargs = []
    cargs.append("mri_create_t2combined")
    cargs.append(subjid)
    cargs.append(execution.input_file(t1wb))
    cargs.append(execution.input_file(t2upper))
    if t2middle is not None:
        cargs.append(execution.input_file(t2middle))
    cargs.append(execution.input_file(t2lower))
    cargs.append(t2combined)
    if show:
        cargs.append("show")
    ret = MriCreateT2combinedOutputs(
        root=execution.output_file("."),
        logfile=execution.output_file("/root/mri_create_t2combined.log"),
        t2combined_output=execution.output_file(t2combined),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "MRI_CREATE_T2COMBINED_METADATA",
    "MriCreateT2combinedOutputs",
    "mri_create_t2combined",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

REGISTER_CHILD_METADATA = Metadata(
    id="9e8cb1a2f3c7d3f5cb4a97d68638d4d529a433fd.boutiques",
    name="register_child",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class RegisterChildOutputs(typing.NamedTuple):
    """
    Output object returned when calling `register_child(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    transformed_control_points: OutputPathType
    """File where transformed control points are written."""
    intensity_normalized_volume: OutputPathType
    """File where intensity normalized volume is written."""


def register_child(
    input_volume: InputPathType,
    output_directory: str,
    runner: Runner | None = None,
) -> RegisterChildOutputs:
    """
    A tool used for registering MR volumes with a child's atlas in Freesurfer.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        input_volume: Input MR volume to be used for registration.
        output_directory: Directory where output files will be written.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `RegisterChildOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(REGISTER_CHILD_METADATA)
    cargs = []
    cargs.append("register_child")
    cargs.append(execution.input_file(input_volume))
    cargs.append(output_directory)
    ret = RegisterChildOutputs(
        root=execution.output_file("."),
        transformed_control_points=execution.output_file(output_directory + "/fsamples"),
        intensity_normalized_volume=execution.output_file(output_directory + "/norm"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "REGISTER_CHILD_METADATA",
    "RegisterChildOutputs",
    "register_child",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

REINFLATE_SUBJECT_RH_METADATA = Metadata(
    id="46c17d80d09536f705ed323c939acc704434405c.boutiques",
    name="reinflate_subject-rh",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class ReinflateSubjectRhOutputs(typing.NamedTuple):
    """
    Output object returned when calling `reinflate_subject_rh(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def reinflate_subject_rh(
    subject_dir: str,
    additional_options: str | None = None,
    runner: Runner | None = None,
) -> ReinflateSubjectRhOutputs:
    """
    A tool for reinflating the cortical surfaces for a given subject in FreeSurfer,
    specifically for the right hemisphere.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        subject_dir: Directory of the subject within FreeSurfer environment.
        additional_options: Additional options for the reinflate_subject-rh\
            command.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `ReinflateSubjectRhOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(REINFLATE_SUBJECT_RH_METADATA)
    cargs = []
    cargs.extend([
        "-rh",
        "reinflate_subject" + subject_dir
    ])
    if additional_options is not None:
        cargs.append(additional_options)
    ret = ReinflateSubjectRhOutputs(
        root=execution.output_file("."),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "REINFLATE_SUBJECT_RH_METADATA",
    "ReinflateSubjectRhOutputs",
    "reinflate_subject_rh",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

V_3DSVM_METADATA = Metadata(
    id="561addc870b41a8529b3ab4135131a6bc7b9089a.boutiques",
    name="3dsvm",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class V3dsvmOutputs(typing.NamedTuple):
    """
    Output object returned when calling `v_3dsvm(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    out_model: OutputPathType
    """Output model file in .1D format."""
    out_alpha: OutputPathType
    """Output alphas file in .1D format."""
    out_bucket: OutputPathType
    """Output bucket file in .1D format."""
    out_predictions: OutputPathType
    """Output predictions file in .1D format."""


def v_3dsvm(
    model: str,
    train_vol: InputPathType | None = None,
    train_labels: InputPathType | None = None,
    mask: InputPathType | None = None,
    no_model_mask: bool = False,
    alpha: str | None = None,
    bucket: str | None = None,
    type_: typing.Literal["classification", "regression"] | None = None,
    c_value: float | None = None,
    epsilon: float | None = None,
    kernel: typing.Literal["linear", "polynomial", "rbf", "sigmoid"] | None = None,
    d_value: float | None = None,
    gamma: float | None = None,
    s_value: float | None = None,
    r_value: float | None = None,
    max_iterations: float | None = None,
    wout: bool = False,
    test_vol: InputPathType | None = None,
    predictions: str | None = None,
    classout: bool = False,
    nopred_censored: bool = False,
    nodetrend: bool = False,
    nopred_scale: bool = False,
    test_labels: InputPathType | None = None,
    multiclass: typing.Literal["DAG", "vote"] | None = None,
    help_: bool = False,
    version: bool = False,
    runner: Runner | None = None,
) -> V3dsvmOutputs:
    """
    Support vector machine analysis of brain data using the SVM-light package.
    
    Author: AFNI Developers
    
    URL: https://afni.nimh.nih.gov/
    
    Args:
        model: The basename for the model brik containing the SVM model during\
            training or testing.
        train_vol: A 3D+t AFNI brik dataset to be used for training.
        train_labels: Filename of class category .1D labels corresponding to\
            the stimulus paradigm for the training data set.
        mask: Specify a mask dataset to only perform the analysis on non-zero\
            mask voxels.
        no_model_mask: Flag to enable the omission of a mask file. Required if\
            '-mask' is not used.
        alpha: Write the alphas to a specified .1D file.
        bucket: Outputs the sum of weighted linear support vectors written out\
            to a functional (fim) brik file.
        type_: Specify type: classification (default) or regression.
        c_value: Control SVM model complexity (C value).
        epsilon: Specify epsilon for regression.
        kernel: Specify type of kernel function.
        d_value: D parameter in polynomial kernel.
        gamma: Gamma parameter in rbf kernel.
        s_value: S parameter in sigmoid/poly kernel.
        r_value: R parameter in sigmoid/poly kernel.
        max_iterations: Specify the maximum number of iterations for the\
            optimization. Default is 1 million.
        wout: Flag to output sum of weighted linear support vectors to the\
            bucket file.
        test_vol: A 3D or 3D+t AFNI brik dataset to be used for testing.
        predictions: Basename for .1D prediction files.
        classout: Flag to specify that prediction files should be\
            integer-valued, corresponding to class category decisions.
        nopred_censored: Do not write predicted values for censored time-points\
            to predictions file.
        nodetrend: Flag to specify that prediction files should NOT be linearly\
            detrended.
        nopred_scale: Do not scale predictions. Values below 0.0 correspond to\
            (class A) and values above 0.0 to (class B).
        test_labels: Filename of 'true' class category .1D labels for the test\
            dataset.
        multiclass: Specify the multiclass algorithm for classification.
        help_: Print help message.
        version: Print version history.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `V3dsvmOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(V_3DSVM_METADATA)
    cargs = []
    cargs.append("3dsvm")
    if train_vol is not None:
        cargs.extend([
            "-trainvol",
            execution.input_file(train_vol)
        ])
    if train_labels is not None:
        cargs.extend([
            "-trainlabels",
            execution.input_file(train_labels)
        ])
    if mask is not None:
        cargs.extend([
            "-mask",
            execution.input_file(mask)
        ])
    if no_model_mask:
        cargs.append("-nomodelmask")
    cargs.extend([
        "-model",
        model
    ])
    if alpha is not None:
        cargs.extend([
            "-alpha",
            alpha
        ])
    if bucket is not None:
        cargs.extend([
            "-bucket",
            bucket
        ])
    if type_ is not None:
        cargs.extend([
            "-type",
            type_
        ])
    if c_value is not None:
        cargs.extend([
            "-c",
            str(c_value)
        ])
    if epsilon is not None:
        cargs.extend([
            "-e",
            str(epsilon)
        ])
    if kernel is not None:
        cargs.extend([
            "-kernel",
            kernel
        ])
    if d_value is not None:
        cargs.extend([
            "-d",
            str(d_value)
        ])
    if gamma is not None:
        cargs.extend([
            "-g",
            str(gamma)
        ])
    if s_value is not None:
        cargs.extend([
            "-s",
            str(s_value)
        ])
    if r_value is not None:
        cargs.extend([
            "-r",
            str(r_value)
        ])
    if max_iterations is not None:
        cargs.extend([
            "-max_iterations",
            str(max_iterations)
        ])
    if wout:
        cargs.append("-wout")
    if test_vol is not None:
        cargs.extend([
            "-testvol",
            execution.input_file(test_vol)
        ])
    if predictions is not None:
        cargs.extend([
            "-predictions",
            predictions
        ])
    if classout:
        cargs.append("-classout")
    if nopred_censored:
        cargs.append("-nopredcensored")
    if nodetrend:
        cargs.append("-nodetrend")
    if nopred_scale:
        cargs.append("-nopredscale")
    if test_labels is not None:
        cargs.extend([
            "-testlabels",
            execution.input_file(test_labels)
        ])
    if multiclass is not None:
        cargs.extend([
            "-multiclass",
            multiclass
        ])
    if help_:
        cargs.append("-help")
    if version:
        cargs.append("-version")
    ret = V3dsvmOutputs(
        root=execution.output_file("."),
        out_model=execution.output_file("model_{output}.1D"),
        out_alpha=execution.output_file("alpha_{output}.1D"),
        out_bucket=execution.output_file("bucket_{output}.1D"),
        out_predictions=execution.output_file("predictions_{output}.1D"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "V3dsvmOutputs",
    "V_3DSVM_METADATA",
    "v_3dsvm",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

MRIS_IMAGE2VTK_METADATA = Metadata(
    id="67a28ca73b630cb9d69525ca86a737dcf4b40ebe.boutiques",
    name="mris_image2vtk",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class MrisImage2vtkOutputs(typing.NamedTuple):
    """
    Output object returned when calling `mris_image2vtk(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_vtk_file: OutputPathType
    """Output VTK file generated by the conversion process"""


def mris_image2vtk(
    input_filename: InputPathType,
    output_filename: str,
    lower_threshold: float,
    upper_threshold: float,
    vtk_smoothing_iters: float,
    image_smoothing_size: float,
    reduction_percent: float,
    runner: Runner | None = None,
) -> MrisImage2vtkOutputs:
    """
    Convert image to VTK format with specified thresholds and smoothing parameters.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        input_filename: Input image file name.
        output_filename: Output VTK file name.
        lower_threshold: Lower threshold for image conversion.
        upper_threshold: Upper threshold for image conversion.
        vtk_smoothing_iters: Number of VTK smoothing iterations.
        image_smoothing_size: Image smoothing size.
        reduction_percent: Reduction percentage for the conversion.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MrisImage2vtkOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(MRIS_IMAGE2VTK_METADATA)
    cargs = []
    cargs.append("mris_image2vtk")
    cargs.append(execution.input_file(input_filename))
    cargs.append(output_filename)
    cargs.append(str(lower_threshold))
    cargs.append(str(upper_threshold))
    cargs.append(str(vtk_smoothing_iters))
    cargs.append(str(image_smoothing_size))
    cargs.append(str(reduction_percent))
    ret = MrisImage2vtkOutputs(
        root=execution.output_file("."),
        output_vtk_file=execution.output_file(output_filename),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "MRIS_IMAGE2VTK_METADATA",
    "MrisImage2vtkOutputs",
    "mris_image2vtk",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

MRI_TOPOLOGYCORRECTION_METADATA = Metadata(
    id="fb398c893ff0a9507f05f5c97dc95a403a3121a3.boutiques",
    name="mri_topologycorrection",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class MriTopologycorrectionOutputs(typing.NamedTuple):
    """
    Output object returned when calling `mri_topologycorrection(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_folder: OutputPathType
    """Directory where output files will be stored."""


def mri_topologycorrection(
    input_orig_file: InputPathType,
    input_segmented_file: InputPathType,
    runner: Runner | None = None,
) -> MriTopologycorrectionOutputs:
    """
    Corrects the topology of segmented MRI images.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        input_orig_file: Path to the original image file.
        input_segmented_file: Path to the segmented image file.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MriTopologycorrectionOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(MRI_TOPOLOGYCORRECTION_METADATA)
    cargs = []
    cargs.append("mri_topologycorrection")
    cargs.append(execution.input_file(input_orig_file))
    cargs.append(execution.input_file(input_segmented_file))
    ret = MriTopologycorrectionOutputs(
        root=execution.output_file("."),
        output_folder=execution.output_file("[OUTPUT_FOLDER]"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "MRI_TOPOLOGYCORRECTION_METADATA",
    "MriTopologycorrectionOutputs",
    "mri_topologycorrection",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

MKSURFATLAS_METADATA = Metadata(
    id="e6c489c83a055f531b66d649615579c36de7f9af.boutiques",
    name="mksurfatlas",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class MksurfatlasOutputs(typing.NamedTuple):
    """
    Output object returned when calling `mksurfatlas(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_atlas: OutputPathType
    """Resulting atlas file in TIFF format."""


def mksurfatlas(
    atlas: str,
    hemi: str,
    subjects: list[str],
    surfval: str,
    surfvaldir: str | None = "label",
    regsurf: str | None = "sphere",
    debug: bool = False,
    version: bool = False,
    help_: bool = False,
    runner: Runner | None = None,
) -> MksurfatlasOutputs:
    """
    Creates an atlas using mris_make_template. The atlas can then be used to create
    the surface registration for each subject based on this atlas.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        atlas: Save results to this file (tif file).
        hemi: Hemisphere to process.
        subjects: Subject(s) to process. Multiple subjects can be specified by\
            repeating the flag.
        surfval: Surface values file. Looks for\
            subject/surfvaldir/hemi.surfval.
        surfvaldir: Directory for surface values; default is 'label'.
        regsurf: Registration surface; default is 'sphere'.
        debug: Turn on debugging.
        version: Print version and exit.
        help_: Print help and exit.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MksurfatlasOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(MKSURFATLAS_METADATA)
    cargs = []
    cargs.append("mksurfatlas")
    cargs.extend([
        "--a",
        atlas
    ])
    cargs.extend([
        "--h",
        hemi
    ])
    cargs.extend([
        "--s",
        *subjects
    ])
    cargs.extend([
        "--v",
        surfval
    ])
    if surfvaldir is not None:
        cargs.extend([
            "--d",
            surfvaldir
        ])
    if regsurf is not None:
        cargs.extend([
            "--r",
            regsurf
        ])
    if debug:
        cargs.append("--debug")
    if version:
        cargs.append("--version")
    if help_:
        cargs.append("--help")
    ret = MksurfatlasOutputs(
        root=execution.output_file("."),
        output_atlas=execution.output_file(atlas),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "MKSURFATLAS_METADATA",
    "MksurfatlasOutputs",
    "mksurfatlas",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

V_3D_DIFF_METADATA = Metadata(
    id="310e8ce2db4e828d08b7d1bb6c0af6eff1fa85cd.boutiques",
    name="3dDiff",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class V3dDiffOutputs(typing.NamedTuple):
    """
    Output object returned when calling `v_3d_diff(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_log: OutputPathType
    """Log file containing the element-wise differences."""


def v_3d_diff(
    dataset_a: InputPathType,
    dataset_b: InputPathType,
    tolerance: float | None = None,
    mask: InputPathType | None = None,
    quiet_mode: bool = False,
    tabular_mode: bool = False,
    brutalist_mode: bool = False,
    long_report_mode: bool = False,
    runner: Runner | None = None,
) -> V3dDiffOutputs:
    """
    A program to examine element-wise differences between two images.
    
    Author: AFNI Developers
    
    URL: https://afni.nimh.nih.gov/
    
    Args:
        dataset_a: First input dataset for comparison.
        dataset_b: Second input dataset for comparison.
        tolerance: Floating-point tolerance/epsilon for the comparison.
        mask: Mask to use when comparing the datasets.
        quiet_mode: Quiet mode: 0 for no differences, 1 for differences, -1 for\
            error.
        tabular_mode: Display a table of differences, mainly for 4D data.
        brutalist_mode: Display one-liner with summary of differences.
        long_report_mode: Print a detailed report with more information.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `V3dDiffOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(V_3D_DIFF_METADATA)
    cargs = []
    cargs.append("3dDiff")
    cargs.extend([
        "-a",
        execution.input_file(dataset_a)
    ])
    cargs.extend([
        "-b",
        execution.input_file(dataset_b)
    ])
    if tolerance is not None:
        cargs.extend([
            "-tol",
            str(tolerance)
        ])
    if mask is not None:
        cargs.extend([
            "-mask",
            execution.input_file(mask)
        ])
    if quiet_mode:
        cargs.append("-q")
    if tabular_mode:
        cargs.append("-tabular")
    if brutalist_mode:
        cargs.append("-brutalist")
    if long_report_mode:
        cargs.append("-long_report")
    ret = V3dDiffOutputs(
        root=execution.output_file("."),
        output_log=execution.output_file(pathlib.Path(dataset_a).name + "_vs_" + pathlib.Path(dataset_b).name + ".log"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "V3dDiffOutputs",
    "V_3D_DIFF_METADATA",
    "v_3d_diff",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

MRI_VOLCLUSTER_METADATA = Metadata(
    id="86e5f012e8262b1a51728b36e2469f6a4678ccf5.boutiques",
    name="mri_volcluster",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class MriVolclusterOutputs(typing.NamedTuple):
    """
    Output object returned when calling `mri_volcluster(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_volume: OutputPathType | None
    """Output volume after clustering."""
    output_cluster_number_volume: OutputPathType | None
    """Output cluster number volume."""
    output_binary_mask: OutputPathType | None
    """Final binary mask output."""
    output_label_file: OutputPathType | None
    """Saved cluster as label file."""


def mri_volcluster(
    input_file: InputPathType,
    summary_file: str | None = None,
    output_volid: str | None = None,
    output_cluster_num_volid: str | None = None,
    cwsig_volid: str | None = None,
    pointset_file: str | None = None,
    min_threshold: float | None = None,
    max_threshold: float | None = None,
    sign: str | None = None,
    no_adjust_flag: bool = False,
    match_value: float | None = None,
    cwpval_threshold: float | None = None,
    registration_file: InputPathType | None = None,
    mni152reg_flag: bool = False,
    regheader_subject: str | None = None,
    fsaverage_flag: bool = False,
    frame_number: float | None = None,
    csd_files: list[InputPathType] | None = None,
    cwsig_map: str | None = None,
    vwsig_map: str | None = None,
    max_cwpval_file: str | None = None,
    csdpdf_file: str | None = None,
    csdpdf_only_flag: bool = False,
    fwhm_value: float | None = None,
    fwhm_file: InputPathType | None = None,
    min_size: float | None = None,
    min_size_vox: float | None = None,
    min_distance: float | None = None,
    allow_diag_flag: bool = False,
    bonferroni_number: float | None = None,
    bonferroni_max_number: float | None = None,
    sig2p_max_flag: bool = False,
    gte_flag: bool = False,
    mask_volid: InputPathType | None = None,
    mask_type: str | None = None,
    mask_frame: float | None = None,
    mask_threshold: float | None = None,
    mask_sign: str | None = None,
    mask_invert_flag: bool = False,
    out_mask_volid: str | None = None,
    out_mask_type: str | None = None,
    label_file: str | None = None,
    nlabel_cluster: float | None = None,
    label_base: str | None = None,
    synth_func: str | None = None,
    diagnostic_level: float | None = None,
    fill_params: str | None = None,
    help_flag: bool = False,
    runner: Runner | None = None,
) -> MriVolclusterOutputs:
    """
    A tool for finding clusters in a volume, useful for analyzing MRI data.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        input_file: Source of volume values.
        summary_file: Text summary file.
        output_volid: Output volume ID after clustering.
        output_cluster_num_volid: Output volume ID with cluster number values.
        cwsig_volid: Volume ID for clusterwise significance.
        pointset_file: Create a freeview pointset of the clusters.
        min_threshold: Minimum intensity threshold.
        max_threshold: Maximum intensity threshold.
        sign: Sign for one-sided tests (<abs>, pos, neg).
        no_adjust_flag: Do not adjust threshold for one-tailed tests.
        match_value: Set thmin=matchval-0.5 and thmax=matchval+0.5.
        cwpval_threshold: Require clusters to have cwp < threshold.
        registration_file: For reporting Talairach coordinates.
        mni152reg_flag: Input is in MNI152 space.
        regheader_subject: Use header registration with subject.
        fsaverage_flag: Assume input is in fsaverage space.
        frame_number: Perform cluster analysis on the nth frame (0-based).
        csd_files: Cluster simulation data files.
        cwsig_map: Map of corrected cluster-wise significances.
        vwsig_map: Map of corrected voxel-wise significances.
        max_cwpval_file: Save p-value of the largest cluster.
        csdpdf_file: PDF/CDF of cluster and max significance.
        csdpdf_only_flag: Write CSD PDF file and exit.
        fwhm_value: FWHM in mm^3, forces GRF analysis.
        fwhm_file: Text file with FWHM in mm^3 for GRF.
        min_size: Minimum volume (mm^3) for a cluster.
        min_size_vox: Minimum number of contiguous voxels for a cluster.
        min_distance: Minimum distance between peak clusters.
        allow_diag_flag: Allow diagonal adjacency for contiguity.
        bonferroni_number: Bonferroni correction across spaces.
        bonferroni_max_number: Bonferroni correction applied to maximum.
        sig2p_max_flag: Convert maximum significance to p-value.
        gte_flag: Use >= when computing p-value from CSD.
        mask_volid: Mask volume ID.
        mask_type: File type of mask volume.
        mask_frame: Nth frame of mask to use.
        mask_threshold: Upper threshold for the mask.
        mask_sign: Sign in mask thresholding (<abs>, neg, pos).
        mask_invert_flag: Invert mask after thresholding.
        out_mask_volid: Path for final binary mask.
        out_mask_type: File type for output mask.
        label_file: File to save nth cluster as a label.
        nlabel_cluster: Save the nth cluster as a label.
        label_base: Base name for saving clusters as labels.
        synth_func: Function for synthetic data (uniform, loguniform,\
            gaussian).
        diagnostic_level: Set diagnostic level.
        fill_params: Parameters for fill operation (invol outvol matchval).
        help_flag: Display help message and exit.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MriVolclusterOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(MRI_VOLCLUSTER_METADATA)
    cargs = []
    cargs.append("mri_volcluster")
    cargs.extend([
        "--in",
        execution.input_file(input_file)
    ])
    if summary_file is not None:
        cargs.extend([
            "--sum",
            summary_file
        ])
    if output_volid is not None:
        cargs.extend([
            "--out",
            output_volid
        ])
    if output_cluster_num_volid is not None:
        cargs.extend([
            "--ocn",
            output_cluster_num_volid
        ])
    if cwsig_volid is not None:
        cargs.extend([
            "--cwsig",
            cwsig_volid
        ])
    if pointset_file is not None:
        cargs.extend([
            "--pointset",
            pointset_file
        ])
    if min_threshold is not None:
        cargs.extend([
            "--thmin",
            str(min_threshold)
        ])
    if max_threshold is not None:
        cargs.extend([
            "--thmax",
            str(max_threshold)
        ])
    if sign is not None:
        cargs.extend([
            "--sign",
            sign
        ])
    if no_adjust_flag:
        cargs.append("--no-adjust")
    if match_value is not None:
        cargs.extend([
            "--match",
            str(match_value)
        ])
    if cwpval_threshold is not None:
        cargs.extend([
            "--cwpvalthresh",
            str(cwpval_threshold)
        ])
    if registration_file is not None:
        cargs.extend([
            "--reg",
            execution.input_file(registration_file)
        ])
    if mni152reg_flag:
        cargs.append("--mni152reg")
    if regheader_subject is not None:
        cargs.extend([
            "--regheader",
            regheader_subject
        ])
    if fsaverage_flag:
        cargs.append("--fsaverage")
    if frame_number is not None:
        cargs.extend([
            "--frame",
            str(frame_number)
        ])
    if csd_files is not None:
        cargs.extend([
            "--csd",
            *[execution.input_file(f) for f in csd_files]
        ])
    if cwsig_map is not None:
        cargs.extend([
            "--cwsig",
            cwsig_map
        ])
    if vwsig_map is not None:
        cargs.extend([
            "--vwsig",
            vwsig_map
        ])
    if max_cwpval_file is not None:
        cargs.extend([
            "--maxcwpval",
            max_cwpval_file
        ])
    if csdpdf_file is not None:
        cargs.extend([
            "--csdpdf",
            csdpdf_file
        ])
    if csdpdf_only_flag:
        cargs.append("--csdpdf-only")
    if fwhm_value is not None:
        cargs.extend([
            "--fwhm",
            str(fwhm_value)
        ])
    if fwhm_file is not None:
        cargs.extend([
            "--fwhmdat",
            execution.input_file(fwhm_file)
        ])
    if min_size is not None:
        cargs.extend([
            "--minsize",
            str(min_size)
        ])
    if min_size_vox is not None:
        cargs.extend([
            "--minsizevox",
            str(min_size_vox)
        ])
    if min_distance is not None:
        cargs.extend([
            "--mindist",
            str(min_distance)
        ])
    if allow_diag_flag:
        cargs.append("--allowdiag")
    if bonferroni_number is not None:
        cargs.extend([
            "--bonferroni",
            str(bonferroni_number)
        ])
    if bonferroni_max_number is not None:
        cargs.extend([
            "--bonferroni-max",
            str(bonferroni_max_number)
        ])
    if sig2p_max_flag:
        cargs.append("--sig2p-max")
    if gte_flag:
        cargs.append("--gte")
    if mask_volid is not None:
        cargs.extend([
            "--mask",
            execution.input_file(mask_volid)
        ])
    if mask_type is not None:
        cargs.extend([
            "--mask_type",
            mask_type
        ])
    if mask_frame is not None:
        cargs.extend([
            "--maskframe",
            str(mask_frame)
        ])
    if mask_threshold is not None:
        cargs.extend([
            "--maskthresh",
            str(mask_threshold)
        ])
    if mask_sign is not None:
        cargs.extend([
            "--masksign",
            mask_sign
        ])
    if mask_invert_flag:
        cargs.append("--maskinvert")
    if out_mask_volid is not None:
        cargs.extend([
            "--outmask",
            out_mask_volid
        ])
    if out_mask_type is not None:
        cargs.extend([
            "--outmask_type",
            out_mask_type
        ])
    if label_file is not None:
        cargs.extend([
            "--label",
            label_file
        ])
    if nlabel_cluster is not None:
        cargs.extend([
            "--nlabelcluster",
            str(nlabel_cluster)
        ])
    if label_base is not None:
        cargs.extend([
            "--labelbase",
            label_base
        ])
    if synth_func is not None:
        cargs.extend([
            "--synth",
            synth_func
        ])
    if diagnostic_level is not None:
        cargs.extend([
            "--diag",
            str(diagnostic_level)
        ])
    if fill_params is not None:
        cargs.extend([
            "--fill",
            fill_params
        ])
    if help_flag:
        cargs.append("--help")
    ret = MriVolclusterOutputs(
        root=execution.output_file("."),
        output_volume=execution.output_file(output_volid) if (output_volid is not None) else None,
        output_cluster_number_volume=execution.output_file(output_cluster_num_volid) if (output_cluster_num_volid is not None) else None,
        output_binary_mask=execution.output_file(out_mask_volid) if (out_mask_volid is not None) else None,
        output_label_file=execution.output_file(label_file) if (label_file is not None) else None,
    )
    execution.run(cargs)
    return ret


__all__ = [
    "MRI_VOLCLUSTER_METADATA",
    "MriVolclusterOutputs",
    "mri_volcluster",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

MAKE_RANDOM_TIMING_PY_METADATA = Metadata(
    id="015b2b1c66a3f929dc8b7bc80bb3691f68a641ef.boutiques",
    name="make_random_timing.py",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class MakeRandomTimingPyOutputs(typing.NamedTuple):
    """
    Output object returned when calling `make_random_timing_py(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    stim_output: OutputPathType
    """Stimulus timing output file"""


def make_random_timing_py(
    num_runs: float,
    run_time: list[float],
    num_stim: float,
    num_reps: list[float],
    prefix: str,
    stim_dur: list[float] | None = None,
    across_runs: bool = False,
    max_consec: list[float] | None = None,
    max_rest: float | None = None,
    min_rest: float | None = None,
    not_first: list[str] | None = None,
    not_last: list[str] | None = None,
    offset: float | None = None,
    ordered_stimuli: list[str] | None = None,
    pre_stim_rest: float | None = None,
    post_stim_rest: float | None = None,
    save_3dd_cmd: str | None = None,
    seed: float | None = None,
    stim_labels: list[str] | None = None,
    t_digits: float | None = None,
    t_gran: float | None = None,
    tr: float | None = None,
    tr_locked: bool = False,
    verb: float | None = None,
    show_timing_stats: bool = False,
    runner: Runner | None = None,
) -> MakeRandomTimingPyOutputs:
    """
    Create random stimulus timing files for use with AFNI 3dDeconvolve.
    
    Author: AFNI Developers
    
    URL: https://afni.nimh.nih.gov/
    
    Args:
        num_runs: Set the number of runs.
        run_time: Set the total time per run (in seconds).
        num_stim: Set the number of stimulus classes.
        num_reps: Set the number of repetitions per class (or across runs).
        prefix: Set the prefix for output filenames.
        stim_dur: Set the duration for a single stimulus (in seconds).
        across_runs: Distribute stimuli across all runs at once.
        max_consec: Specify maximum consecutive stimuli per class.
        max_rest: Specify maximum rest between stimuli.
        min_rest: Specify extra rest after each stimulus.
        not_first: Specify classes that should not start a run.
        not_last: Specify classes that should not end a run.
        offset: Specify an offset to add to every stim time.
        ordered_stimuli: Specify a partial ordering of stimuli.
        pre_stim_rest: Specify minimum rest period to start each run.
        post_stim_rest: Specify minimum rest period to end each run.
        save_3dd_cmd: Save a 3dDeconvolve -nodata example.
        seed: Specify a seed for random number generation.
        stim_labels: Specify labels for the stimulus classes.
        t_digits: Set the number of decimal places for times.
        t_gran: Set the time granularity.
        tr: Set the scanner TR.
        tr_locked: Make stimuli timing locked to the accompanying TR.
        verb: Set the verbose level.
        show_timing_stats: Show statistics from the timing.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MakeRandomTimingPyOutputs`).
    """
    if not (1 <= num_runs): 
        raise ValueError(f"'num_runs' must be greater than 1 <= x but was {num_runs}")
    if not (1 <= num_stim): 
        raise ValueError(f"'num_stim' must be greater than 1 <= x but was {num_stim}")
    if not_first is not None and not (1 <= len(not_first)): 
        raise ValueError(f"Length of 'not_first' must be greater than 1 but was {len(not_first)}")
    if not_last is not None and not (1 <= len(not_last)): 
        raise ValueError(f"Length of 'not_last' must be greater than 1 but was {len(not_last)}")
    if ordered_stimuli is not None and not (1 <= len(ordered_stimuli)): 
        raise ValueError(f"Length of 'ordered_stimuli' must be greater than 1 but was {len(ordered_stimuli)}")
    if stim_labels is not None and not (1 <= len(stim_labels)): 
        raise ValueError(f"Length of 'stim_labels' must be greater than 1 but was {len(stim_labels)}")
    runner = runner or get_global_runner()
    execution = runner.start_execution(MAKE_RANDOM_TIMING_PY_METADATA)
    cargs = []
    cargs.append("make_random_timing.py")
    cargs.extend([
        "-num_runs",
        str(num_runs)
    ])
    cargs.extend([
        "-run_time",
        *map(str, run_time)
    ])
    cargs.extend([
        "-num_stim",
        str(num_stim)
    ])
    cargs.extend([
        "-num_reps",
        *map(str, num_reps)
    ])
    cargs.extend([
        "-prefix",
        prefix
    ])
    if stim_dur is not None:
        cargs.extend([
            "-stim_dur",
            *map(str, stim_dur)
        ])
    if across_runs:
        cargs.append("-across_runs")
    if max_consec is not None:
        cargs.extend([
            "-max_consec",
            *map(str, max_consec)
        ])
    if max_rest is not None:
        cargs.extend([
            "-max_rest",
            str(max_rest)
        ])
    if min_rest is not None:
        cargs.extend([
            "-min_rest",
            str(min_rest)
        ])
    if not_first is not None:
        cargs.extend([
            "-not_first",
            *not_first
        ])
    if not_last is not None:
        cargs.extend([
            "-not_last",
            *not_last
        ])
    if offset is not None:
        cargs.extend([
            "-offset",
            str(offset)
        ])
    if ordered_stimuli is not None:
        cargs.extend([
            "-ordered_stimuli",
            *ordered_stimuli
        ])
    if pre_stim_rest is not None:
        cargs.extend([
            "-pre_stim_rest",
            str(pre_stim_rest)
        ])
    if post_stim_rest is not None:
        cargs.extend([
            "-post_stim_rest",
            str(post_stim_rest)
        ])
    if save_3dd_cmd is not None:
        cargs.extend([
            "-save_3dd_cmd",
            save_3dd_cmd
        ])
    if seed is not None:
        cargs.extend([
            "-seed",
            str(seed)
        ])
    if stim_labels is not None:
        cargs.extend([
            "-stim_labels",
            *stim_labels
        ])
    if t_digits is not None:
        cargs.extend([
            "-t_digits",
            str(t_digits)
        ])
    if t_gran is not None:
        cargs.extend([
            "-t_gran",
            str(t_gran)
        ])
    if tr is not None:
        cargs.extend([
            "-tr",
            str(tr)
        ])
    if tr_locked:
        cargs.append("-tr_locked")
    if verb is not None:
        cargs.extend([
            "-verb",
            str(verb)
        ])
    if show_timing_stats:
        cargs.append("-show_timing_stats")
    ret = MakeRandomTimingPyOutputs(
        root=execution.output_file("."),
        stim_output=execution.output_file(prefix + "_*.1D"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "MAKE_RANDOM_TIMING_PY_METADATA",
    "MakeRandomTimingPyOutputs",
    "make_random_timing_py",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

CIFTI_DILATE_METADATA = Metadata(
    id="907e475cef6dbf3556c5c8ea8d183b2219169bf5.boutiques",
    name="cifti-dilate",
    package="workbench",
    container_image_tag="brainlife/connectome_workbench:1.5.0-freesurfer-update",
)


@dataclasses.dataclass
class CiftiDilateLeftSurface:
    """
    specify the left surface to use.
    """
    surface: InputPathType
    """the left surface file"""
    opt_left_corrected_areas_area_metric: InputPathType | None = None
    """vertex areas to use instead of computing them from the left surface: the
    corrected vertex areas, as a metric"""
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        cargs.append("-left-surface")
        cargs.append(execution.input_file(self.surface))
        if self.opt_left_corrected_areas_area_metric is not None:
            cargs.extend([
                "-left-corrected-areas",
                execution.input_file(self.opt_left_corrected_areas_area_metric)
            ])
        return cargs


@dataclasses.dataclass
class CiftiDilateRightSurface:
    """
    specify the right surface to use.
    """
    surface: InputPathType
    """the right surface file"""
    opt_right_corrected_areas_area_metric: InputPathType | None = None
    """vertex areas to use instead of computing them from the right surface: the
    corrected vertex areas, as a metric"""
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        cargs.append("-right-surface")
        cargs.append(execution.input_file(self.surface))
        if self.opt_right_corrected_areas_area_metric is not None:
            cargs.extend([
                "-right-corrected-areas",
                execution.input_file(self.opt_right_corrected_areas_area_metric)
            ])
        return cargs


@dataclasses.dataclass
class CiftiDilateCerebellumSurface:
    """
    specify the cerebellum surface to use.
    """
    surface: InputPathType
    """the cerebellum surface file"""
    opt_cerebellum_corrected_areas_area_metric: InputPathType | None = None
    """vertex areas to use instead of computing them from the cerebellum
    surface: the corrected vertex areas, as a metric"""
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        cargs.append("-cerebellum-surface")
        cargs.append(execution.input_file(self.surface))
        if self.opt_cerebellum_corrected_areas_area_metric is not None:
            cargs.extend([
                "-cerebellum-corrected-areas",
                execution.input_file(self.opt_cerebellum_corrected_areas_area_metric)
            ])
        return cargs


class CiftiDilateOutputs(typing.NamedTuple):
    """
    Output object returned when calling `cifti_dilate(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    cifti_out: OutputPathType
    """the output cifti file"""


def cifti_dilate(
    cifti_in: InputPathType,
    direction: str,
    surface_distance: float,
    volume_distance: float,
    cifti_out: str,
    left_surface: CiftiDilateLeftSurface | None = None,
    right_surface: CiftiDilateRightSurface | None = None,
    cerebellum_surface: CiftiDilateCerebellumSurface | None = None,
    opt_bad_brainordinate_roi_roi_cifti: InputPathType | None = None,
    opt_nearest: bool = False,
    opt_merged_volume: bool = False,
    opt_legacy_mode: bool = False,
    runner: Runner | None = None,
) -> CiftiDilateOutputs:
    """
    Dilate a cifti file.
    
    For all data values designated as bad, if they neighbor a good value or are
    within the specified distance of a good value in the same kind of model,
    replace the value with a distance weighted average of nearby good values,
    otherwise set the value to zero. If -nearest is specified, it will use the
    value from the closest good value within range instead of a weighted
    average. When the input file contains label data, nearest dilation is used
    on the surface, and weighted popularity is used in the volume.
    
    The -*-corrected-areas options are intended for dilating on group average
    surfaces, but it is only an approximate correction for the reduction of
    structure in a group average surface.
    
    If -bad-brainordinate-roi is specified, all values, including those with
    value zero, are good, except for locations with a positive value in the ROI.
    If it is not specified, only values equal to zero are bad.
    
    Author: Connectome Workbench Developers
    
    URL: https://github.com/Washington-University/workbench
    
    Args:
        cifti_in: the input cifti file.
        direction: which dimension to dilate along, ROW or COLUMN.
        surface_distance: the distance to dilate on surfaces, in mm.
        volume_distance: the distance to dilate in the volume, in mm.
        cifti_out: the output cifti file.
        left_surface: specify the left surface to use.
        right_surface: specify the right surface to use.
        cerebellum_surface: specify the cerebellum surface to use.
        opt_bad_brainordinate_roi_roi_cifti: specify an roi of brainordinates\
            to overwrite, rather than zeros: cifti dscalar or dtseries file,\
            positive values denote brainordinates to have their values replaced.
        opt_nearest: use nearest good value instead of a weighted average.
        opt_merged_volume: treat volume components as if they were a single\
            component.
        opt_legacy_mode: use the math from v1.3.2 and earlier for weighted\
            dilation.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CiftiDilateOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(CIFTI_DILATE_METADATA)
    cargs = []
    cargs.append("wb_command")
    cargs.append("-cifti-dilate")
    cargs.append(execution.input_file(cifti_in))
    cargs.append(direction)
    cargs.append(str(surface_distance))
    cargs.append(str(volume_distance))
    cargs.append(cifti_out)
    if left_surface is not None:
        cargs.extend(left_surface.run(execution))
    if right_surface is not None:
        cargs.extend(right_surface.run(execution))
    if cerebellum_surface is not None:
        cargs.extend(cerebellum_surface.run(execution))
    if opt_bad_brainordinate_roi_roi_cifti is not None:
        cargs.extend([
            "-bad-brainordinate-roi",
            execution.input_file(opt_bad_brainordinate_roi_roi_cifti)
        ])
    if opt_nearest:
        cargs.append("-nearest")
    if opt_merged_volume:
        cargs.append("-merged-volume")
    if opt_legacy_mode:
        cargs.append("-legacy-mode")
    ret = CiftiDilateOutputs(
        root=execution.output_file("."),
        cifti_out=execution.output_file(cifti_out),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "CIFTI_DILATE_METADATA",
    "CiftiDilateCerebellumSurface",
    "CiftiDilateLeftSurface",
    "CiftiDilateOutputs",
    "CiftiDilateRightSurface",
    "cifti_dilate",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

TBSS_2_REG_METADATA = Metadata(
    id="8e75d5ab55c5f140a059acbd323eb9f4d32ea205.boutiques",
    name="tbss_2_reg",
    package="fsl",
    container_image_tag="brainlife/fsl:6.0.4-patched2",
)


class Tbss2RegOutputs(typing.NamedTuple):
    """
    Output object returned when calling `tbss_2_reg(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def tbss_2_reg(
    use_fmrib58_fa_1mm: bool = False,
    target_image: InputPathType | None = None,
    find_best_target: bool = False,
    runner: Runner | None = None,
) -> Tbss2RegOutputs:
    """
    TBSS utility for target selection and registration for Tract-Based Spatial
    Statistics (TBSS) analysis.
    
    Author: FMRIB Analysis Group, University of Oxford
    
    URL: https://fsl.fmrib.ox.ac.uk/fsl/fslwiki
    
    Args:
        use_fmrib58_fa_1mm: Use FMRIB58_FA_1mm as the target for nonlinear\
            registrations (recommended).
        target_image: Use the specified image as the target for nonlinear\
            registrations.
        find_best_target: Find the best target from all images in the FA.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `Tbss2RegOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(TBSS_2_REG_METADATA)
    cargs = []
    cargs.append("tbss_2_reg")
    if use_fmrib58_fa_1mm:
        cargs.append("-T")
    if target_image is not None:
        cargs.extend([
            "-t",
            execution.input_file(target_image)
        ])
    if find_best_target:
        cargs.append("-n")
    ret = Tbss2RegOutputs(
        root=execution.output_file("."),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "TBSS_2_REG_METADATA",
    "Tbss2RegOutputs",
    "tbss_2_reg",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

FS_CHECK_VERSION_METADATA = Metadata(
    id="1bb29df8432be8183122dba0b74d19ba214f7751.boutiques",
    name="fs-check-version",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class FsCheckVersionOutputs(typing.NamedTuple):
    """
    Output object returned when calling `fs_check_version(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_file: OutputPathType
    """Output text file with a 1 if the version matches, otherwise 0"""


def fs_check_version(
    subjects_dir: str,
    outfile: str,
    subject: str | None = None,
    no_require_match: bool = False,
    test_debug: bool = False,
    runner: Runner | None = None,
) -> FsCheckVersionOutputs:
    """
    Script to manage which version of FreeSurfer can be used to analyze data
    ensuring consistency with the desired version.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        subjects_dir: Subjects directory path.
        outfile: Output file path where result of version check will be written.
        subject: Subject name (optional).
        no_require_match: Unset REQUIRE_FS_MATCH for testing.
        test_debug: Go through permutations for debugging.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `FsCheckVersionOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(FS_CHECK_VERSION_METADATA)
    cargs = []
    cargs.append("fs-check-version")
    cargs.extend([
        "--sd",
        subjects_dir
    ])
    cargs.extend([
        "--o",
        outfile
    ])
    if subject is not None:
        cargs.extend([
            "--s",
            subject
        ])
    if no_require_match:
        cargs.append("--no-require-match")
    if test_debug:
        cargs.append("--test-debug")
    ret = FsCheckVersionOutputs(
        root=execution.output_file("."),
        output_file=execution.output_file(outfile),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "FS_CHECK_VERSION_METADATA",
    "FsCheckVersionOutputs",
    "fs_check_version",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

MP_DIFFPOW_METADATA = Metadata(
    id="5d6f81008119f2f4a31e758201b79a4cc2a55918.boutiques",
    name="mp_diffpow",
    package="fsl",
    container_image_tag="brainlife/fsl:6.0.4-patched2",
)


class MpDiffpowOutputs(typing.NamedTuple):
    """
    Output object returned when calling `mp_diffpow(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    outfile: OutputPathType
    """File containing squared motion parameters, temporal difference of motion
    parameters, and squared differenced values."""


def mp_diffpow(
    reg_file: InputPathType,
    diff_reg_file: str,
    runner: Runner | None = None,
) -> MpDiffpowOutputs:
    """
    Generates a file with specific motion parameter calculations useful for
    accounting for 'spin history' effects and other variations not accounted for by
    motion correction.
    
    Author: FMRIB Analysis Group, University of Oxford
    
    URL: https://fsl.fmrib.ox.ac.uk/fsl/fslwiki
    
    Args:
        reg_file: Input file containing registration parameters (e.g.,\
            regparam.dat).
        diff_reg_file: Output file with differenced registration parameters\
            (e.g., diffregparam.dat).
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MpDiffpowOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(MP_DIFFPOW_METADATA)
    cargs = []
    cargs.append("mp_diffpow.sh")
    cargs.append(execution.input_file(reg_file))
    cargs.append(diff_reg_file)
    ret = MpDiffpowOutputs(
        root=execution.output_file("."),
        outfile=execution.output_file(diff_reg_file),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "MP_DIFFPOW_METADATA",
    "MpDiffpowOutputs",
    "mp_diffpow",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

V__RETINO_PROC_METADATA = Metadata(
    id="0836ad099b5ae0ebd794fce9bafc7b29723fb572.boutiques",
    name="@RetinoProc",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class VRetinoProcOutputs(typing.NamedTuple):
    """
    Output object returned when calling `v__retino_proc(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def v__retino_proc(
    tr: float,
    period_ecc: float,
    period_pol: float,
    ccw: list[InputPathType] | None = None,
    clw: list[InputPathType] | None = None,
    exp: list[InputPathType] | None = None,
    con: list[InputPathType] | None = None,
    epi_ref: InputPathType | None = None,
    epi_anat_ref: InputPathType | None = None,
    anat_vol: InputPathType | None = None,
    anat_vol_epi: InputPathType | None = None,
    surf_vol: InputPathType | None = None,
    surf_vol_epi: InputPathType | None = None,
    phase: bool = False,
    delay: bool = False,
    pre_ecc: float | None = None,
    pre_pol: float | None = None,
    on_ecc: str | None = None,
    on_pol: str | None = None,
    var_on_ecc: str | None = None,
    var_on_pol: str | None = None,
    nwedges: float | None = None,
    nrings: float | None = None,
    fwhm_pol: float | None = None,
    fwhm_ecc: float | None = None,
    ignore: float | None = None,
    no_tshift: bool = False,
    spec_left: InputPathType | None = None,
    spec_right: InputPathType | None = None,
    dorts: InputPathType | None = None,
    ccw_orts: list[InputPathType] | None = None,
    clw_orts: list[InputPathType] | None = None,
    exp_orts: list[InputPathType] | None = None,
    con_orts: list[InputPathType] | None = None,
    sid: str | None = None,
    out_dir: str | None = None,
    echo: bool = False,
    echo_edu: bool = False,
    a2e_opts: str | None = None,
    aea_opts: str | None = None,
    runner: Runner | None = None,
) -> VRetinoProcOutputs:
    """
    A script to process retinotopic FMRI data, using AFNI's 3dRetinoPhase and
    SurfRetinMap.
    
    Author: AFNI Developers
    
    URL: https://afni.nimh.nih.gov/
    
    Args:
        tr: TR, in seconds, of retinotopic scans.
        period_ecc: Period, in seconds, of eccentricity stimuli.
        period_pol: Period, in seconds, of polar angle stimuli.
        ccw: Input time series dataset for counterclockwise stimulus.
        clw: Input time series dataset for clockwise stimulus.
        exp: Input time series dataset for expanding stimulus.
        con: Input time series dataset for contracting stimulus.
        epi_ref: Specify a volume from the EPI time series to which all EPI\
            volumes are aligned.
        epi_anat_ref: Specify a volume from the EPI time series that is better\
            suited for aligning the T1 to it than EpiRef might be.
        anat_vol: T1 volume acquired during the same session as the retinotopic\
            scans.
        anat_vol_epi: Anatomical volume aligned to EPI reference.
        surf_vol: Surface Volume for the cortical surfaces.
        surf_vol_epi: Surface volume aligned to experiment's EPI data.
        phase: Use phase of fundamental frequency to estimate latency.
        delay: Use delay relative to reference time series to estimate latency.
        pre_ecc: Duration, in seconds, before eccentricity stimulus.
        pre_pol: Duration, in seconds, before polar angle stimulus.
        on_ecc: Number of stimulation blocks and duration of stimulation for\
            eccentricity stimulus.
        on_pol: Number of stimulation blocks and duration of stimulation for\
            polar angle stimulus.
        var_on_ecc: Multiple on durations for eccentricity stimulus.
        var_on_pol: Multiple on durations for polar angle stimulus.
        nwedges: Number of wedges in polar stimulus.
        nrings: Number of rings in eccentricity stimulus.
        fwhm_pol: Target smoothness for polar stimulus.
        fwhm_ecc: Target smoothness for eccentricity stimulus.
        ignore: Ignore volumes from the beginning of each time series.
        no_tshift: Do not correct for slice timing.
        spec_left: Spec file for left hemisphere.
        spec_right: Spec file for right hemisphere.
        dorts: Detrend time series using columns in ORT1D file.
        ccw_orts: Detrend time series for counterclockwise stimulus.
        clw_orts: Detrend time series for clockwise stimulus.
        exp_orts: Detrend time series for expanding stimulus.
        con_orts: Detrend time series for contracting stimulus.
        sid: SID is a flag identifying the subject.
        out_dir: Directory where processing results are to be stored.
        echo: Turn on the command echoing to help with debugging script failure.
        echo_edu: Turn on command echoing for certain programs only.
        a2e_opts: Pass options to @SUMA_AlignToExperiment script.
        aea_opts: Pass options to align_epi_anat.py.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VRetinoProcOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(V__RETINO_PROC_METADATA)
    cargs = []
    cargs.append("@RetinoProc")
    if ccw is not None:
        cargs.extend([
            "-ccw",
            *[execution.input_file(f) for f in ccw]
        ])
    if clw is not None:
        cargs.extend([
            "-clw",
            *[execution.input_file(f) for f in clw]
        ])
    if exp is not None:
        cargs.extend([
            "-exp",
            *[execution.input_file(f) for f in exp]
        ])
    if con is not None:
        cargs.extend([
            "-con",
            *[execution.input_file(f) for f in con]
        ])
    if epi_ref is not None:
        cargs.extend([
            "-epi_ref",
            execution.input_file(epi_ref)
        ])
    if epi_anat_ref is not None:
        cargs.extend([
            "-epi_anat_ref",
            execution.input_file(epi_anat_ref)
        ])
    if anat_vol is not None:
        cargs.extend([
            "-anat_vol",
            execution.input_file(anat_vol)
        ])
    if anat_vol_epi is not None:
        cargs.extend([
            "-anat_vol@epi",
            execution.input_file(anat_vol_epi)
        ])
    if surf_vol is not None:
        cargs.extend([
            "-surf_vol",
            execution.input_file(surf_vol)
        ])
    if surf_vol_epi is not None:
        cargs.extend([
            "-surf_vol@epi",
            execution.input_file(surf_vol_epi)
        ])
    if phase:
        cargs.append("-phase")
    if delay:
        cargs.append("-delay")
    cargs.extend([
        "-TR",
        str(tr)
    ])
    cargs.extend([
        "-period_ecc",
        str(period_ecc)
    ])
    cargs.extend([
        "-period_pol",
        str(period_pol)
    ])
    if pre_ecc is not None:
        cargs.extend([
            "-pre_ecc",
            str(pre_ecc)
        ])
    if pre_pol is not None:
        cargs.extend([
            "-pre_pol",
            str(pre_pol)
        ])
    if on_ecc is not None:
        cargs.extend([
            "-on_ecc",
            on_ecc
        ])
    if on_pol is not None:
        cargs.extend([
            "-on_pol",
            on_pol
        ])
    if var_on_ecc is not None:
        cargs.extend([
            "-var_on_ecc",
            var_on_ecc
        ])
    if var_on_pol is not None:
        cargs.extend([
            "-var_on_pol",
            var_on_pol
        ])
    if nwedges is not None:
        cargs.extend([
            "-nwedges",
            str(nwedges)
        ])
    if nrings is not None:
        cargs.extend([
            "-nrings",
            str(nrings)
        ])
    if fwhm_pol is not None:
        cargs.extend([
            "-fwhm_pol",
            str(fwhm_pol)
        ])
    if fwhm_ecc is not None:
        cargs.extend([
            "-fwhm_ecc",
            str(fwhm_ecc)
        ])
    if ignore is not None:
        cargs.extend([
            "-ignore",
            str(ignore)
        ])
    if no_tshift:
        cargs.append("-no_tshift")
    if spec_left is not None:
        cargs.extend([
            "-spec_left",
            execution.input_file(spec_left)
        ])
    if spec_right is not None:
        cargs.extend([
            "-spec_right",
            execution.input_file(spec_right)
        ])
    if dorts is not None:
        cargs.extend([
            "-dorts",
            execution.input_file(dorts)
        ])
    if ccw_orts is not None:
        cargs.extend([
            "-ccw_orts",
            *[execution.input_file(f) for f in ccw_orts]
        ])
    if clw_orts is not None:
        cargs.extend([
            "-clw_orts",
            *[execution.input_file(f) for f in clw_orts]
        ])
    if exp_orts is not None:
        cargs.extend([
            "-exp_orts",
            *[execution.input_file(f) for f in exp_orts]
        ])
    if con_orts is not None:
        cargs.extend([
            "-con_orts",
            *[execution.input_file(f) for f in con_orts]
        ])
    if sid is not None:
        cargs.extend([
            "-sid",
            sid
        ])
    if out_dir is not None:
        cargs.extend([
            "-out_dir",
            out_dir
        ])
    if echo:
        cargs.append("-echo")
    if echo_edu:
        cargs.append("-echo_edu")
    if a2e_opts is not None:
        cargs.extend([
            "-A2E_opts",
            a2e_opts
        ])
    if aea_opts is not None:
        cargs.extend([
            "-AEA_opts",
            aea_opts
        ])
    ret = VRetinoProcOutputs(
        root=execution.output_file("."),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "VRetinoProcOutputs",
    "V__RETINO_PROC_METADATA",
    "v__retino_proc",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

LONGMC_METADATA = Metadata(
    id="51a04d5ffd0bb81591249e8acfb084306bc3edcb.boutiques",
    name="longmc",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class LongmcOutputs(typing.NamedTuple):
    """
    Output object returned when calling `longmc(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def longmc(
    cross_tp_name: str,
    base_name: str,
    subjects_dir: str,
    conform_to_hires: bool = False,
    subject_name: str | None = None,
    no_force_update: bool = False,
    runner: Runner | None = None,
) -> LongmcOutputs:
    """
    Perform motion correction for the longitudinal recon-all stream when creating
    the longitudinal timepoint of a subject.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        cross_tp_name: Cross time point name for the longitudinal analysis.
        base_name: Base name for the longitudinal analysis.
        subjects_dir: Set the SUBJECTS_DIR directory.
        conform_to_hires: Option to conform input to high-resolution.
        subject_name: Subject name override, must be declared after -long.
        no_force_update: Do not force update.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `LongmcOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(LONGMC_METADATA)
    cargs = []
    cargs.append("longmc")
    cargs.extend([
        "-long",
        cross_tp_name
    ])
    cargs.append(base_name)
    if conform_to_hires:
        cargs.append("-conf2hires")
    cargs.extend([
        "-sd",
        subjects_dir
    ])
    if subject_name is not None:
        cargs.extend([
            "-s",
            subject_name
        ])
    if no_force_update:
        cargs.append("-no-force-update")
    ret = LongmcOutputs(
        root=execution.output_file("."),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "LONGMC_METADATA",
    "LongmcOutputs",
    "longmc",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

V_3D_LOCALSTAT_METADATA = Metadata(
    id="f927c9d4ad8b0ef0fa566fe0b9648789d0b3eb44.boutiques",
    name="3dLocalstat",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class V3dLocalstatOutputs(typing.NamedTuple):
    """
    Output object returned when calling `v_3d_localstat(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    outfile: OutputPathType | None
    """Output dataset"""


def v_3d_localstat(
    dataset: InputPathType,
    nbhd: str,
    stat_: list[str] | None = None,
    mask: InputPathType | None = None,
    automask: bool = False,
    use_nonmask: bool = False,
    prefix: str | None = None,
    datum: str | None = None,
    label_ext: str | None = None,
    reduce_grid: list[float] | None = None,
    reduce_restore_grid: list[float] | None = None,
    reduce_max_vox: float | None = None,
    grid_rmode: str | None = None,
    quiet: bool = False,
    verbose: bool = False,
    proceed_small_n: bool = False,
    fillvalue: float | None = None,
    unfillvalue: float | None = None,
    maskvalue: float | None = None,
    maskvalue2: float | None = None,
    runner: Runner | None = None,
) -> V3dLocalstatOutputs:
    """
    This program computes statistics at each voxel, based on a local neighborhood of
    that voxel.
    
    Author: AFNI Developers
    
    URL: https://afni.nimh.nih.gov/
    
    Args:
        dataset: Input dataset.
        nbhd: The region around each voxel that will be extracted for the\
            statistics calculation.
        stat_: Compute the specified statistic on the values extracted from the\
            neighborhood.
        mask: Read in dataset 'mset' and use the nonzero voxels therein as a\
            mask.
        automask: Compute the mask as in program 3dAutomask (mutually exclusive\
            with -mask).
        use_nonmask: Compute local statistics from all voxels in the\
            neighborhood that are in the mask, even if the central voxel is not in\
            the mask.
        prefix: Use the given string as the prefix for the output dataset.
        datum: Coerce the output data to be stored as the given type (byte,\
            short, float).
        label_ext: Append given label to each sub-brick label.
        reduce_grid: Compute output on a grid that is reduced by the given\
            factor in X, Y, and Z directions of the input dataset.
        reduce_restore_grid: Resample the output back to input grid after\
            reducing the grid.
        reduce_max_vox: Automatically set Rx Ry Rz so that the computation grid\
            is at a resolution of nbhd/MAX_VOX voxels.
        grid_rmode: Interpolant to use when resampling the output with\
            reduce_restore_grid option.
        quiet: Stop the highly informative progress reports.
        verbose: A little more verbose output.
        proceed_small_n: Do not crash if neighborhood is too small for certain\
            estimates.
        fillvalue: Value used for filled statistic, default=1.
        unfillvalue: Value used for unfilled statistic, default=1.
        maskvalue: Value searched for with has_mask option.
        maskvalue2: Alternate value for has_mask2 option.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `V3dLocalstatOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(V_3D_LOCALSTAT_METADATA)
    cargs = []
    cargs.append("3dLocalstat")
    cargs.append(execution.input_file(dataset))
    cargs.extend([
        "-nbhd",
        nbhd
    ])
    if stat_ is not None:
        cargs.extend([
            "-stat",
            *stat_
        ])
    if mask is not None:
        cargs.extend([
            "-mask",
            execution.input_file(mask)
        ])
    if automask:
        cargs.append("-automask")
    if use_nonmask:
        cargs.append("-use_nonmask")
    if prefix is not None:
        cargs.extend([
            "-prefix",
            prefix
        ])
    if datum is not None:
        cargs.extend([
            "-datum",
            datum
        ])
    if label_ext is not None:
        cargs.extend([
            "-label_ext",
            label_ext
        ])
    if reduce_grid is not None:
        cargs.extend([
            "-reduce_grid",
            *map(str, reduce_grid)
        ])
    if reduce_restore_grid is not None:
        cargs.extend([
            "-reduce_restore_grid",
            *map(str, reduce_restore_grid)
        ])
    if reduce_max_vox is not None:
        cargs.extend([
            "-reduce_max_vox",
            str(reduce_max_vox)
        ])
    if grid_rmode is not None:
        cargs.extend([
            "-grid_rmode",
            grid_rmode
        ])
    if quiet:
        cargs.append("-quiet")
    if verbose:
        cargs.append("-verb")
    if proceed_small_n:
        cargs.append("-proceed_small_N")
    if fillvalue is not None:
        cargs.extend([
            "-fillvalue",
            str(fillvalue)
        ])
    if unfillvalue is not None:
        cargs.extend([
            "-unfillvalue",
            str(unfillvalue)
        ])
    if maskvalue is not None:
        cargs.extend([
            "-maskvalue",
            str(maskvalue)
        ])
    if maskvalue2 is not None:
        cargs.extend([
            "-maskvalue2",
            str(maskvalue2)
        ])
    ret = V3dLocalstatOutputs(
        root=execution.output_file("."),
        outfile=execution.output_file(prefix + ".nii.gz") if (prefix is not None) else None,
    )
    execution.run(cargs)
    return ret


__all__ = [
    "V3dLocalstatOutputs",
    "V_3D_LOCALSTAT_METADATA",
    "v_3d_localstat",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

MS_REFINE_SUBJECT_METADATA = Metadata(
    id="c522667edafe63ffe7ee067f5ceb07e59d920d4c.boutiques",
    name="ms_refine_subject",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class MsRefineSubjectOutputs(typing.NamedTuple):
    """
    Output object returned when calling `ms_refine_subject(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def ms_refine_subject(
    subjects_dir: str,
    runner: Runner | None = None,
) -> MsRefineSubjectOutputs:
    """
    Unknown.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        subjects_dir: Directory containing the subject data (e.g.\
            /usr/local/freesurfer/subjects).
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MsRefineSubjectOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(MS_REFINE_SUBJECT_METADATA)
    cargs = []
    cargs.append("ms_refine_subject")
    cargs.append(subjects_dir)
    ret = MsRefineSubjectOutputs(
        root=execution.output_file("."),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "MS_REFINE_SUBJECT_METADATA",
    "MsRefineSubjectOutputs",
    "ms_refine_subject",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

MRIS_SURFACE_TO_VOL_DISTANCES_METADATA = Metadata(
    id="b436cdee4792f571a6fb7c9b7311696958c3dc2d.boutiques",
    name="mris_surface_to_vol_distances",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class MrisSurfaceToVolDistancesOutputs(typing.NamedTuple):
    """
    Output object returned when calling `mris_surface_to_vol_distances(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_distances: OutputPathType
    """Output file containing surface-to-volume distances."""


def mris_surface_to_vol_distances(
    average_subject: str,
    hemisphere: str,
    subjects: list[str],
    output_prefix: str,
    runner: Runner | None = None,
) -> MrisSurfaceToVolDistancesOutputs:
    """
    Tool from FreeSurfer to calculate surface-to-volume distances.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        average_subject: The average subject.
        hemisphere: Hemisphere identifier (e.g., lh or rh).
        subjects: List of subjects for distance calculation.
        output_prefix: Prefix for output files.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MrisSurfaceToVolDistancesOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(MRIS_SURFACE_TO_VOL_DISTANCES_METADATA)
    cargs = []
    cargs.append("mris_surface_to_vol_distances")
    cargs.append(average_subject)
    cargs.append(hemisphere)
    cargs.extend(subjects)
    cargs.append(output_prefix)
    ret = MrisSurfaceToVolDistancesOutputs(
        root=execution.output_file("."),
        output_distances=execution.output_file(output_prefix + "_distances.txt"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "MRIS_SURFACE_TO_VOL_DISTANCES_METADATA",
    "MrisSurfaceToVolDistancesOutputs",
    "mris_surface_to_vol_distances",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

V_1D_TOOL_PY_METADATA = Metadata(
    id="14f0b3bf351835bacda04353b7e54c019220ebc2.boutiques",
    name="1d_tool.py",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class V1dToolPyOutputs(typing.NamedTuple):
    """
    Output object returned when calling `v_1d_tool_py(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    outfile: OutputPathType | None
    """Resulting 1D file"""


def v_1d_tool_py(
    infile: InputPathType,
    write: str | None = None,
    select_cols: str | None = None,
    select_rows: str | None = None,
    select_groups: str | None = None,
    censor_motion: float | None = None,
    pad_into_many_runs: str | None = None,
    set_nruns: float | None = None,
    set_run_lengths: str | None = None,
    show_rows_cols: bool = False,
    transpose: bool = False,
    reverse: bool = False,
    show_max_displace: bool = False,
    runner: Runner | None = None,
) -> V1dToolPyOutputs:
    """
    A tool for manipulating and evaluating 1D files.
    
    Author: AFNI Developers
    
    URL: https://afni.nimh.nih.gov/
    
    Args:
        infile: Input 1D file.
        write: Output file to write results.
        select_cols: Select specific columns.
        select_rows: Select specific rows.
        select_groups: Select columns by group numbers.
        censor_motion: Generate a boolean censor file.
        pad_into_many_runs: Pad a 1D file into many runs.
        set_nruns: Set number of runs.
        set_run_lengths: Set run lengths.
        show_rows_cols: Show the number of rows and columns.
        transpose: Transpose the input matrix.
        reverse: Reverse the data over time.
        show_max_displace: Show the maximum pairwise displacement.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `V1dToolPyOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(V_1D_TOOL_PY_METADATA)
    cargs = []
    cargs.append("1d_tool.py")
    cargs.extend([
        "-infile",
        execution.input_file(infile)
    ])
    if write is not None:
        cargs.extend([
            "-write",
            write
        ])
    if select_cols is not None:
        cargs.extend([
            "-select_cols",
            select_cols
        ])
    if select_rows is not None:
        cargs.extend([
            "-select_rows",
            select_rows
        ])
    if select_groups is not None:
        cargs.extend([
            "-select_groups",
            select_groups
        ])
    if censor_motion is not None:
        cargs.extend([
            "-censor_motion",
            str(censor_motion)
        ])
    if pad_into_many_runs is not None:
        cargs.extend([
            "-pad_into_many_runs",
            pad_into_many_runs
        ])
    if set_nruns is not None:
        cargs.extend([
            "-set_nruns",
            str(set_nruns)
        ])
    if set_run_lengths is not None:
        cargs.extend([
            "-set_run_lengths",
            set_run_lengths
        ])
    if show_rows_cols:
        cargs.append("-show_rows_cols")
    if transpose:
        cargs.append("-transpose")
    if reverse:
        cargs.append("-reverse")
    if show_max_displace:
        cargs.append("-show_max_displace")
    ret = V1dToolPyOutputs(
        root=execution.output_file("."),
        outfile=execution.output_file(write) if (write is not None) else None,
    )
    execution.run(cargs)
    return ret


__all__ = [
    "V1dToolPyOutputs",
    "V_1D_TOOL_PY_METADATA",
    "v_1d_tool_py",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

UPDATE_NEEDED_METADATA = Metadata(
    id="81ad1ec598b447ced529bab2b15819d597e08ec9.boutiques",
    name="UpdateNeeded",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class UpdateNeededOutputs(typing.NamedTuple):
    """
    Output object returned when calling `update_needed(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def update_needed(
    target_file: InputPathType,
    source_file: InputPathType,
    runner: Runner | None = None,
) -> UpdateNeededOutputs:
    """
    A command-line tool to update a target file based on one or more source files.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        target_file: The target file that needs to be updated.
        source_file: The primary source file for updating the target file.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `UpdateNeededOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(UPDATE_NEEDED_METADATA)
    cargs = []
    cargs.append("UpdateNeeded")
    cargs.append(execution.input_file(target_file))
    cargs.append(execution.input_file(source_file))
    cargs.append("[ADDITIONAL_SOURCE_FILES...]")
    ret = UpdateNeededOutputs(
        root=execution.output_file("."),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "UPDATE_NEEDED_METADATA",
    "UpdateNeededOutputs",
    "update_needed",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

VOLUME_VECTOR_OPERATION_METADATA = Metadata(
    id="659c860de35d61543458fd69cdcc014c15293fe8.boutiques",
    name="volume-vector-operation",
    package="workbench",
    container_image_tag="brainlife/connectome_workbench:1.5.0-freesurfer-update",
)


class VolumeVectorOperationOutputs(typing.NamedTuple):
    """
    Output object returned when calling `volume_vector_operation(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    volume_out: OutputPathType
    """the output file"""


def volume_vector_operation(
    vectors_a: InputPathType,
    vectors_b: InputPathType,
    operation: str,
    volume_out: str,
    opt_normalize_a: bool = False,
    opt_normalize_b: bool = False,
    opt_normalize_output: bool = False,
    opt_magnitude: bool = False,
    runner: Runner | None = None,
) -> VolumeVectorOperationOutputs:
    """
    Do a vector operation on volume files.
    
    Does a vector operation on two volume files (that must have a multiple of 3
    subvolumes). Either of the inputs may have multiple vectors (more than 3
    subvolumes), but not both (at least one must have exactly 3 subvolumes). The
    -magnitude and -normalize-output options may not be specified together, or
    with the DOT operation. The <operation> parameter must be one of the
    following:
    
    DOT
    CROSS
    ADD
    SUBTRACT.
    
    Author: Connectome Workbench Developers
    
    URL: https://github.com/Washington-University/workbench
    
    Args:
        vectors_a: first vector input file.
        vectors_b: second vector input file.
        operation: what vector operation to do.
        volume_out: the output file.
        opt_normalize_a: normalize vectors of first input.
        opt_normalize_b: normalize vectors of second input.
        opt_normalize_output: normalize output vectors (not valid for dot\
            product).
        opt_magnitude: output the magnitude of the result (not valid for dot\
            product).
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VolumeVectorOperationOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(VOLUME_VECTOR_OPERATION_METADATA)
    cargs = []
    cargs.append("wb_command")
    cargs.append("-volume-vector-operation")
    cargs.append(execution.input_file(vectors_a))
    cargs.append(execution.input_file(vectors_b))
    cargs.append(operation)
    cargs.append(volume_out)
    if opt_normalize_a:
        cargs.append("-normalize-a")
    if opt_normalize_b:
        cargs.append("-normalize-b")
    if opt_normalize_output:
        cargs.append("-normalize-output")
    if opt_magnitude:
        cargs.append("-magnitude")
    ret = VolumeVectorOperationOutputs(
        root=execution.output_file("."),
        volume_out=execution.output_file(volume_out),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "VOLUME_VECTOR_OPERATION_METADATA",
    "VolumeVectorOperationOutputs",
    "volume_vector_operation",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

MRI_VOLSYNTH_METADATA = Metadata(
    id="656047848a60982264a81504c030cd41366f6765.boutiques",
    name="mri_volsynth",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class MriVolsynthOutputs(typing.NamedTuple):
    """
    Output object returned when calling `mri_volsynth(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def mri_volsynth(
    output_volid: str,
    template: str | None = None,
    nframes: float | None = None,
    offset_flag: bool = False,
    offset_mid_flag: bool = False,
    curv: str | None = None,
    dim: list[float] | None = None,
    res: list[float] | None = None,
    vox_size: list[float] | None = None,
    tr: float | None = None,
    cdircos: list[float] | None = None,
    rdircos: list[float] | None = None,
    sdircos: list[float] | None = None,
    c_ras: list[float] | None = None,
    p0: list[float] | None = None,
    precision: str | None = None,
    seed: float | None = None,
    seedfile: InputPathType | None = None,
    pdf: str | None = None,
    bb: list[float] | None = None,
    gmean: float | None = None,
    gstd: float | None = None,
    delta_crsf: list[float] | None = None,
    delta_val: float | None = None,
    delta_val_off: float | None = None,
    grid: list[float] | None = None,
    dof: float | None = None,
    dof_num: float | None = None,
    dof_den: float | None = None,
    rescale_flag: bool = False,
    val_a: float | None = None,
    val_b: float | None = None,
    vox_radius: float | None = None,
    mm_radius: float | None = None,
    sphere_center: list[float] | None = None,
    hsc: list[float] | None = None,
    abs_flag: bool = False,
    cp: InputPathType | None = None,
    spike: float | None = None,
    fwhm: float | None = None,
    sum2: InputPathType | None = None,
    dim_surf_flag: bool = False,
    ctab: InputPathType | None = None,
    runner: Runner | None = None,
) -> MriVolsynthOutputs:
    """
    Synthesizes a volume with specified geometry and probability distribution
    function.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        output_volid: Output volume path id and format.
        template: Template volume id.
        nframes: Override template number of frames.
        offset_flag: Use template as intensity offset.
        offset_mid_flag: Use middle frame of template as intensity offset.
        curv: Save output as curvature, uses lh.thickness as template. Requires\
            subject and hemisphere.
        dim: Specify dimensionality nc nr ns nf.
        res: Voxel resolution dc dr ds df (df is TR, in msec).
        vox_size: Change template voxel resolution and dimension dc dr ds.
        tr: Time between frames in msec.
        cdircos: Column cosine direction x, y, z.
        rdircos: Row cosine direction x, y, z.
        sdircos: Slice cosine direction x, y, z.
        c_ras: RAS coordinates of 'center' voxel c_r c_a c_s.
        p0: First voxel coordinates p0r p0a p0s.
        precision: Precision of the output (e.g., float).
        seed: Seed for the random number generator.
        seedfile: Write seed value to this file.
        pdf: Probability distribution function (e.g., gaussian, uniform, const).
        bb: Bounding box c r s dc dr ds (In=ValA, Out=ValB).
        gmean: Mean for the gaussian distribution.
        gstd: Standard deviation for the gaussian distribution.
        delta_crsf: Delta's col, row, slice, and frame coordinates.
        delta_val: Delta value.
        delta_val_off: Delta background value.
        grid: Grid dimensions dcol, drow, dslice.
        dof: Degrees of freedom for t and chi2 distributions.
        dof_num: Numerator degrees of freedom for F distribution.
        dof_den: Denominator degrees of freedom for F distribution.
        rescale_flag: Rescale z, t, F, or chi2 after smoothing.
        val_a: Set ValA.
        val_b: Set ValB.
        vox_radius: Radius in voxels for sphere.
        mm_radius: Radius in mm for sphere.
        sphere_center: Sphere center coordinates column, row, slice.
        hsc: Multiply each frame by a random number between min and max.
        abs_flag: Compute absolute value.
        cp: Set control point voxels to 1.
        spike: Set all values at a given time point to 1e9.
        fwhm: Smooth by Full Width at Half Maximum (FWHM) in mm.
        sum2: Save sum of volume squared into specified file.
        dim_surf_flag: Set dimension to nvertices x 1 x 1.
        ctab: Embed color table.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MriVolsynthOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(MRI_VOLSYNTH_METADATA)
    cargs = []
    cargs.append("mri_volsynth")
    cargs.extend([
        "--o",
        output_volid
    ])
    if template is not None:
        cargs.extend([
            "--template",
            template
        ])
    if nframes is not None:
        cargs.extend([
            "--nframes",
            str(nframes)
        ])
    if offset_flag:
        cargs.append("--offset")
    if offset_mid_flag:
        cargs.append("--offset-mid")
    if curv is not None:
        cargs.extend([
            "--curv",
            curv
        ])
    if dim is not None:
        cargs.extend([
            "--dim",
            *map(str, dim)
        ])
    if res is not None:
        cargs.extend([
            "--res",
            *map(str, res)
        ])
    if vox_size is not None:
        cargs.extend([
            "--vox-size",
            *map(str, vox_size)
        ])
    if tr is not None:
        cargs.extend([
            "--tr",
            str(tr)
        ])
    if cdircos is not None:
        cargs.extend([
            "--cdircos",
            *map(str, cdircos)
        ])
    if rdircos is not None:
        cargs.extend([
            "--rdircos",
            *map(str, rdircos)
        ])
    if sdircos is not None:
        cargs.extend([
            "--sdircos",
            *map(str, sdircos)
        ])
    if c_ras is not None:
        cargs.extend([
            "--c_ras",
            *map(str, c_ras)
        ])
    if p0 is not None:
        cargs.extend([
            "--p0",
            *map(str, p0)
        ])
    if precision is not None:
        cargs.extend([
            "--precision",
            precision
        ])
    if seed is not None:
        cargs.extend([
            "--seed",
            str(seed)
        ])
    if seedfile is not None:
        cargs.extend([
            "--seedfile",
            execution.input_file(seedfile)
        ])
    if pdf is not None:
        cargs.extend([
            "--pdf",
            pdf
        ])
    if bb is not None:
        cargs.extend([
            "--bb",
            *map(str, bb)
        ])
    if gmean is not None:
        cargs.extend([
            "--gmean",
            str(gmean)
        ])
    if gstd is not None:
        cargs.extend([
            "--gstd",
            str(gstd)
        ])
    if delta_crsf is not None:
        cargs.extend([
            "--delta-crsf",
            *map(str, delta_crsf)
        ])
    if delta_val is not None:
        cargs.extend([
            "--delta-val",
            str(delta_val)
        ])
    if delta_val_off is not None:
        cargs.extend([
            "--delta-val-off",
            str(delta_val_off)
        ])
    if grid is not None:
        cargs.extend([
            "--grid",
            *map(str, grid)
        ])
    if dof is not None:
        cargs.extend([
            "--dof",
            str(dof)
        ])
    if dof_num is not None:
        cargs.extend([
            "--dof-num",
            str(dof_num)
        ])
    if dof_den is not None:
        cargs.extend([
            "--dof-den",
            str(dof_den)
        ])
    if rescale_flag:
        cargs.append("--rescale")
    if val_a is not None:
        cargs.extend([
            "--val-a",
            str(val_a)
        ])
    if val_b is not None:
        cargs.extend([
            "--val-b",
            str(val_b)
        ])
    if vox_radius is not None:
        cargs.extend([
            "--vox-radius",
            str(vox_radius)
        ])
    if mm_radius is not None:
        cargs.extend([
            "--mm-radius",
            str(mm_radius)
        ])
    if sphere_center is not None:
        cargs.extend([
            "--sphere-center",
            *map(str, sphere_center)
        ])
    if hsc is not None:
        cargs.extend([
            "--hsc",
            *map(str, hsc)
        ])
    if abs_flag:
        cargs.append("--abs")
    if cp is not None:
        cargs.extend([
            "--cp",
            execution.input_file(cp)
        ])
    if spike is not None:
        cargs.extend([
            "--spike",
            str(spike)
        ])
    if fwhm is not None:
        cargs.extend([
            "--fwhm",
            str(fwhm)
        ])
    if sum2 is not None:
        cargs.extend([
            "--sum2",
            execution.input_file(sum2)
        ])
    if dim_surf_flag:
        cargs.append("--dim-surf")
    if ctab is not None:
        cargs.extend([
            "--ctab",
            execution.input_file(ctab)
        ])
    ret = MriVolsynthOutputs(
        root=execution.output_file("."),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "MRI_VOLSYNTH_METADATA",
    "MriVolsynthOutputs",
    "mri_volsynth",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

CONVEX_HULL_METADATA = Metadata(
    id="945c8df244e1356a8e523e12b46e0aa912eddb4c.boutiques",
    name="ConvexHull",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class ConvexHullOutputs(typing.NamedTuple):
    """
    Output object returned when calling `convex_hull(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    out_surf: OutputPathType | None
    """Output surface file"""


def convex_hull(
    vol: InputPathType | None = None,
    isoval: float | None = None,
    isorange: list[float] | None = None,
    isocmask: str | None = None,
    xform: str | None = None,
    surface_input: InputPathType | None = None,
    surf_vol: InputPathType | None = None,
    input_1d: InputPathType | None = None,
    q_opt: str | None = None,
    proj_xy: bool = False,
    orig_coord: bool = False,
    these_coords: InputPathType | None = None,
    output_prefix: str | None = None,
    debug: str | None = None,
    novolreg: bool = False,
    setenv: str | None = None,
    runner: Runner | None = None,
) -> ConvexHullOutputs:
    """
    A program to find the convex hull, or perform a Delaunay triangulation of a set
    of points.
    
    Author: AFNI Developers
    
    URL: https://afni.nimh.nih.gov/
    
    Args:
        vol: Input AFNI (or AFNI readable) volume.
        isoval: Create isosurface where volume = V.
        isorange: Create isosurface where V0 <= volume < V1.
        isocmask: Create isosurface where MASK_COM != 0. Example: -isocmask '-a\
            VOL+orig -expr (1-bool(a-V))' is equivalent to using -isoval V. NOTE:\
            Allowed only with -xform mask.
        xform: Transform to apply to volume values before searching for sign\
            change boundary. Options: mask, shift, none.
        surface_input: Input surface type.
        surf_vol: Specify a surface volume which contains a transform to apply\
            to the surface node coordinates.
        input_1d: Construct the triangulation of the points contained in 1D\
            file XYZ. Use AFNI's [] selectors to specify the XYZ columns.
        q_opt: Meshing option OPT. Options: convex_hull, triangulate_xy.
        proj_xy: Project points onto plane whose normal is the third principal\
            component. Then rotate projection so that plane is parallel to Z =\
            constant.
        orig_coord: Use original coordinates when writing surface, not\
            transformed ones.
        these_coords: Use coordinates in COORDS.1D when writing surface.
        output_prefix: Prefix of output surface. Specifies the format and\
            prefix of the surface.
        debug: Debugging level.
        novolreg: Ignore any Rotate, Volreg, Tagalign, or WarpDrive\
            transformations present in the Surface Volume.
        setenv: Set environment variable ENVname to be ENVvalue.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `ConvexHullOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(CONVEX_HULL_METADATA)
    cargs = []
    cargs.append("ConvexHull")
    if vol is not None:
        cargs.extend([
            "-input",
            execution.input_file(vol)
        ])
    if isoval is not None:
        cargs.extend([
            "-isoval",
            str(isoval)
        ])
    if isorange is not None:
        cargs.extend([
            "-isorange",
            *map(str, isorange)
        ])
    if isocmask is not None:
        cargs.extend([
            "-isocmask",
            isocmask
        ])
    if xform is not None:
        cargs.extend([
            "-xform",
            xform
        ])
    if surface_input is not None:
        cargs.extend([
            "-i_TYPE",
            execution.input_file(surface_input)
        ])
    if surf_vol is not None:
        cargs.extend([
            "-sv",
            execution.input_file(surf_vol)
        ])
    if input_1d is not None:
        cargs.extend([
            "-input_1D",
            execution.input_file(input_1d)
        ])
    if q_opt is not None:
        cargs.extend([
            "-q_opt",
            q_opt
        ])
    if proj_xy:
        cargs.append("-proj_xy")
    if orig_coord:
        cargs.append("-orig_coord")
    if these_coords is not None:
        cargs.extend([
            "-these_coords",
            execution.input_file(these_coords)
        ])
    if output_prefix is not None:
        cargs.extend([
            "-o_TYPE",
            output_prefix
        ])
    if debug is not None:
        cargs.extend([
            "-debug",
            debug
        ])
    if novolreg:
        cargs.append("-novolreg")
    if setenv is not None:
        cargs.extend([
            "-setenv",
            setenv
        ])
    ret = ConvexHullOutputs(
        root=execution.output_file("."),
        out_surf=execution.output_file(output_prefix) if (output_prefix is not None) else None,
    )
    execution.run(cargs)
    return ret


__all__ = [
    "CONVEX_HULL_METADATA",
    "ConvexHullOutputs",
    "convex_hull",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

BASIL_VAR_METADATA = Metadata(
    id="ef2d05e150b63d8d6121c52d86fea7583edc0338.boutiques",
    name="basil_var",
    package="fsl",
    container_image_tag="brainlife/fsl:6.0.4-patched2",
)


class BasilVarOutputs(typing.NamedTuple):
    """
    Output object returned when calling `basil_var(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def basil_var(
    results_dir: str,
    mask_image: InputPathType,
    runner: Runner | None = None,
) -> BasilVarOutputs:
    """
    Variance calculator for BASIL.
    
    Author: FMRIB Analysis Group, University of Oxford
    
    URL: https://fsl.fmrib.ox.ac.uk/fsl/fslwiki
    
    Args:
        results_dir: BASIL results directory.
        mask_image: Mask image.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `BasilVarOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(BASIL_VAR_METADATA)
    cargs = []
    cargs.append("basil_var")
    cargs.extend([
        "-d",
        results_dir
    ])
    cargs.extend([
        "-m",
        execution.input_file(mask_image)
    ])
    ret = BasilVarOutputs(
        root=execution.output_file("."),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "BASIL_VAR_METADATA",
    "BasilVarOutputs",
    "basil_var",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

V__ATLASIZE_METADATA = Metadata(
    id="5fc4c62439ac62644ef483caa2fc7a4ea1cb35f0.boutiques",
    name="@Atlasize",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class VAtlasizeOutputs(typing.NamedTuple):
    """
    Output object returned when calling `v__atlasize(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    niml_file: OutputPathType | None
    """Generated NIML file for the atlas"""


def v__atlasize(
    dset: InputPathType | None = None,
    space: str | None = None,
    lab_file: list[str] | None = None,
    lab_file_delim: str | None = None,
    longnames: float | None = None,
    last_longname_col: float | None = None,
    atlas_type: str | None = None,
    atlas_description: str | None = None,
    atlas_name: str | None = None,
    auto_backup: bool = False,
    centers: bool = False,
    centertype: str | None = None,
    centermask: InputPathType | None = None,
    skip_novoxels: bool = False,
    h_web: bool = False,
    h_view: bool = False,
    all_opts: bool = False,
    h_find: str | None = None,
    runner: Runner | None = None,
) -> VAtlasizeOutputs:
    """
    Script to turn a volumetric dataset into an AFNI atlas.
    
    Author: AFNI Developers
    
    URL: https://afni.nimh.nih.gov/
    
    Args:
        dset: Make DSET an atlas.
        space: Mark DSET as being in space SPACE.
        lab_file: Labels and keys are in text file FILE. cLAB is the index of\
            column containing labels, vVAL is the index of column containing keys\
            (1st column is indexed at 0).
        lab_file_delim: Set column delimiter for -lab_file option. Default is '\
            ' (space), but you can set your own. ';' for example.
        longnames: Additionally, allow for another column of long names for\
            regions, e.g., amygdala for AMY. cLONGNAME is the starting column for\
            the long name continuing to the last name of the output.
        last_longname_col: Limit long names to nth column.
        atlas_type: Set the atlas type where TP is 'S' for subject-based and\
            'G' for group-based atlases, respectively.
        atlas_description: A description for the atlas. Default is 'My Atlas'.
        atlas_name: Name for the atlas. Default name is based on prefix of\
            DSET.
        auto_backup: Back up the dataset if it already exists in the custom\
            atlas directory and allows an overwrite.
        centers: Add center of mass coordinates to atlas.
        centertype: Choose Icent, Dcent, or cm for different ways to compute\
            centers.
        centermask: Calculate center of mass locations for each ROI using a\
            subset of voxels. Useful for atlases with identical labels in both\
            hemispheres.
        skip_novoxels: Skip regions without any voxels in the dataset.
        h_web: Open webpage with help for this program.
        h_view: Open -help output in a GUI editor.
        all_opts: List all of the options for this script.
        h_find: Search for lines containing WORD in -help output. Search is\
            approximate.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VAtlasizeOutputs`).
    """
    if lab_file is not None and not (len(lab_file) <= 3): 
        raise ValueError(f"Length of 'lab_file' must be less than 3 but was {len(lab_file)}")
    runner = runner or get_global_runner()
    execution = runner.start_execution(V__ATLASIZE_METADATA)
    cargs = []
    cargs.append("@Atlasize")
    if dset is not None:
        cargs.extend([
            "-dset",
            execution.input_file(dset)
        ])
    if space is not None:
        cargs.extend([
            "-space",
            space
        ])
    if lab_file is not None:
        cargs.extend([
            "-lab_file",
            *lab_file
        ])
    if lab_file_delim is not None:
        cargs.extend([
            "-lab_file_delim",
            lab_file_delim
        ])
    if longnames is not None:
        cargs.extend([
            "-longnames",
            str(longnames)
        ])
    if last_longname_col is not None:
        cargs.extend([
            "-last_longname_col",
            str(last_longname_col)
        ])
    if atlas_type is not None:
        cargs.extend([
            "-atlas_type",
            atlas_type
        ])
    if atlas_description is not None:
        cargs.extend([
            "-atlas_description",
            atlas_description
        ])
    if atlas_name is not None:
        cargs.extend([
            "-atlas_name",
            atlas_name
        ])
    if auto_backup:
        cargs.append("-auto_backup")
    if centers:
        cargs.append("-centers")
    if centertype is not None:
        cargs.extend([
            "-centertype",
            centertype
        ])
    if centermask is not None:
        cargs.extend([
            "-centermask",
            execution.input_file(centermask)
        ])
    if skip_novoxels:
        cargs.append("-skip_novoxels")
    if h_web:
        cargs.append("-h_web")
    if h_view:
        cargs.append("-h_view")
    if all_opts:
        cargs.append("-all_opts")
    if h_find is not None:
        cargs.extend([
            "-h_find",
            h_find
        ])
    ret = VAtlasizeOutputs(
        root=execution.output_file("."),
        niml_file=execution.output_file(pathlib.Path(dset).name + ".niml") if (dset is not None) else None,
    )
    execution.run(cargs)
    return ret


__all__ = [
    "VAtlasizeOutputs",
    "V__ATLASIZE_METADATA",
    "v__atlasize",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

V_3D_TSPLIT4_D_METADATA = Metadata(
    id="861535b6c29e168191a0da4498bda4b743f855bf.boutiques",
    name="3dTsplit4D",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class V3dTsplit4DOutputs(typing.NamedTuple):
    """
    Output object returned when calling `v_3d_tsplit4_d(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    outfiles: OutputPathType
    """Multiple 3D single-brick output files"""


def v_3d_tsplit4_d(
    prefix: str,
    infile: InputPathType,
    keep_datum: bool = False,
    digits: float | None = None,
    runner: Runner | None = None,
) -> V3dTsplit4DOutputs:
    """
    Convert a 3D+time dataset into multiple 3D single-brick files.
    
    Author: AFNI Developers
    
    URL: https://afni.nimh.nih.gov/
    
    Args:
        prefix: Prefix of the output datasets (e.g., out/epi).
        infile: Input 3D+time dataset (e.g., epi_r1+orig).
        keep_datum: Output uses original datum (no conversion to float).
        digits: Number of digits to use for output filenames.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `V3dTsplit4DOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(V_3D_TSPLIT4_D_METADATA)
    cargs = []
    cargs.append("3dTsplit4D")
    cargs.extend([
        "-prefix",
        prefix
    ])
    cargs.append(execution.input_file(infile))
    if keep_datum:
        cargs.append("-keep_datum")
    if digits is not None:
        cargs.extend([
            "-digits",
            str(digits)
        ])
    ret = V3dTsplit4DOutputs(
        root=execution.output_file("."),
        outfiles=execution.output_file(prefix + "*"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "V3dTsplit4DOutputs",
    "V_3D_TSPLIT4_D_METADATA",
    "v_3d_tsplit4_d",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

DMRI_SAVE_HISTOGRAMS_METADATA = Metadata(
    id="ee8d1ca09c1b2241dc168157801f106029c44920.boutiques",
    name="dmri_saveHistograms",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class DmriSaveHistogramsOutputs(typing.NamedTuple):
    """
    Output object returned when calling `dmri_save_histograms(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    histogram_csv: OutputPathType
    """CSV file containing the output histograms."""


def dmri_save_histograms(
    parcellation: InputPathType,
    number_of_bundles: float,
    vtk_bundle_list: list[InputPathType],
    output_csv: str,
    brain_bundle_flag: bool = False,
    runner: Runner | None = None,
) -> DmriSaveHistogramsOutputs:
    """
    A tool to save histograms from diffusion MRI tractography data.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        parcellation: Parcellation file for the tractography data.
        number_of_bundles: Number of bundles in the tractography data.
        vtk_bundle_list: List of VTK bundles for creating histograms.
        output_csv: Output CSV file to save histograms.
        brain_bundle_flag: Brain Bundle flag.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `DmriSaveHistogramsOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(DMRI_SAVE_HISTOGRAMS_METADATA)
    cargs = []
    cargs.append("dmri_saveHistograms")
    cargs.extend([
        "-p",
        execution.input_file(parcellation)
    ])
    cargs.extend([
        "-f",
        str(number_of_bundles)
    ])
    cargs.extend([execution.input_file(f) for f in vtk_bundle_list])
    cargs.extend([
        "-o",
        output_csv
    ])
    if brain_bundle_flag:
        cargs.append("-bb")
    ret = DmriSaveHistogramsOutputs(
        root=execution.output_file("."),
        histogram_csv=execution.output_file("histograms.csv"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "DMRI_SAVE_HISTOGRAMS_METADATA",
    "DmriSaveHistogramsOutputs",
    "dmri_save_histograms",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

MRIS_DIVIDE_PARCELLATION_METADATA = Metadata(
    id="973cad6ea7ea439576bea5f4ba09e31fd5af1401.boutiques",
    name="mris_divide_parcellation",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class MrisDivideParcellationOutputs(typing.NamedTuple):
    """
    Output object returned when calling `mris_divide_parcellation(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    outannot_file: OutputPathType
    """The resulting annotation file with divided labels."""


def mris_divide_parcellation(
    subject: str,
    hemi: str,
    sourceannot: InputPathType,
    splitfile_or_areathresh: str,
    outannot: str,
    scale: float | None = None,
    label_name: str | None = None,
    runner: Runner | None = None,
) -> MrisDivideParcellationOutputs:
    """
    Divides one or more parcellations into divisions perpendicular to the long axis
    of the label.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        subject: The subject identifier.
        hemi: The hemisphere identifier, e.g., left (lh) or right (rh).
        sourceannot: The source annotation file.
        splitfile_or_areathresh: Either a splitfile specifying divisions or an\
            area threshold in mm^2.
        outannot: The output annotation file name.
        scale: Specify offset scaling for rgb values (default=20).
        label_name: Only process the label <label name> (not implemented yet).
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MrisDivideParcellationOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(MRIS_DIVIDE_PARCELLATION_METADATA)
    cargs = []
    cargs.append("mris_divide_parcellation")
    cargs.append(subject)
    cargs.append(hemi)
    cargs.append(execution.input_file(sourceannot))
    cargs.append(splitfile_or_areathresh)
    cargs.append(outannot)
    if scale is not None:
        cargs.extend([
            "-scale",
            str(scale)
        ])
    if label_name is not None:
        cargs.extend([
            "-l",
            label_name
        ])
    ret = MrisDivideParcellationOutputs(
        root=execution.output_file("."),
        outannot_file=execution.output_file(outannot),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "MRIS_DIVIDE_PARCELLATION_METADATA",
    "MrisDivideParcellationOutputs",
    "mris_divide_parcellation",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

MRI_CA_TISSUE_PARMS_METADATA = Metadata(
    id="1d6429470b2b62de7212566259b4407543b03a79.boutiques",
    name="mri_ca_tissue_parms",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class MriCaTissueParmsOutputs(typing.NamedTuple):
    """
    Output object returned when calling `mri_ca_tissue_parms(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_file: OutputPathType
    """File containing the computed tissue parameters."""


def mri_ca_tissue_parms(
    subjects: list[str],
    output_file: str,
    spacing_flag: bool = False,
    gradient_flag: bool = False,
    runner: Runner | None = None,
) -> MriCaTissueParmsOutputs:
    """
    Tool for computing tissue parameters in canonical space.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        subjects: List of subjects for processing.
        output_file: Output file for storing results.
        spacing_flag: Specify spacing of classifiers in canonical space.
        gradient_flag: Use intensity gradient as input to classifier.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MriCaTissueParmsOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(MRI_CA_TISSUE_PARMS_METADATA)
    cargs = []
    cargs.append("mri_ca_tissue_parms")
    cargs.extend(subjects)
    cargs.append(output_file)
    if spacing_flag:
        cargs.append("-spacing")
    if gradient_flag:
        cargs.append("-gradient")
    ret = MriCaTissueParmsOutputs(
        root=execution.output_file("."),
        output_file=execution.output_file(output_file),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "MRI_CA_TISSUE_PARMS_METADATA",
    "MriCaTissueParmsOutputs",
    "mri_ca_tissue_parms",
]
